package com.huoran.iasf.common.utils;

import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.common.exception.code.ResponseCodeInterface;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * 返回值R
 *
 * @author cheney
 * @version V1.0
 * @date 2022年7月28日
 */
@Data
public class R {

    /**
     * 请求响应code，0为成功 其他为失败
     */
    @ApiModelProperty(value = "请求响应code，200为成功 其他为失败", name = "code")
    private int code;

    /**
     * 响应异常码详细信息
     */
    @ApiModelProperty(value = "响应异常码详细信息", name = "msg")
    private String msg;

    @ApiModelProperty(value = "需要返回的数据", name = "data")
    private Object data;

    public R(int code, Object data) {
        this.code = code;
        this.data = data;
        this.msg = null;
    }

    public R(int code, String msg, Object data) {
        this.code = code;
        this.msg = msg;
        this.data = data;
    }

    public R(int code, String msg) {
        this.code = code;
        this.msg = msg;
        this.data = null;
    }

    public R() {
        this.code = BaseResponseCode.SUCCESS.getCode();
        this.msg = BaseResponseCode.SUCCESS.getMsg();
        this.data = null;
    }

    public R(Object data) {
        this.data = data;
        this.code = BaseResponseCode.SUCCESS.getCode();
        this.msg = BaseResponseCode.SUCCESS.getMsg();
    }

    public R(ResponseCodeInterface responseCodeInterface) {
        this.data = null;
        this.code = responseCodeInterface.getCode();
        this.msg = responseCodeInterface.getMsg();
    }

    public R(ResponseCodeInterface responseCodeInterface, Object data) {
        this.data = data;
        this.code = responseCodeInterface.getCode();
        this.msg = responseCodeInterface.getMsg();
    }

    /**
     * 操作成功 data为null
     */
    public static R success() {
        return new R();
    }

    /**
     * 操作成功 data 不为null
     */
    public static R success(Object data) {
        return new R(data);
    }

    /**
     * 操作失败 data 不为null
     */
    public static R fail(String msg) {
        return new R(BaseResponseCode.OPERATION_ERROR.getCode(), msg);
    }

    public static R fail(Object data) {
        return new R(data);
    }


    /**
     * 自定义返回  data为null
     */
    public static R getResult(int code, String msg) {
        return new R(code, msg);
    }

    /**
     * 自定义返回 入参一般是异常code枚举 data为空
     */
    public static R getResult(BaseResponseCode responseCode) {
        return new R(responseCode);
    }


    /**
     * 返回警告信息的R对象，通常用于前端展示警告但不完全阻止操作的场景。
     *
     * @param warnMsg 警告信息
     * @return 包含警告信息的R对象实例
     */
    public static R warn(String warnMsg) {
        // 这里直接使用字符串信息作为警告，如果您有专门的警告响应码，可以使用如下形式：
        // return new R(BaseResponseCode.WARNING_CODE, warnMsg);

        // 或者，如果只想传递消息而不改变响应码（保持成功响应码，但附带警告信息）:
        return new R(300, warnMsg);
    }


    public static R result(boolean delState, String s) {
        return delState ? success(s) : fail(s);
    }

    public static R success(Object data, String message) {
        return new R(200, message, data);
    }

}
