package com.huoran.iasf.controller;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.huoran.iasf.common.aop.annotation.NoRepeatSubmit;
import com.huoran.iasf.common.exception.NotFoundException;
import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.common.utils.Constant;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.entity.*;
import com.huoran.iasf.service.*;
import com.huoran.iasf.vo.req.ColumnWeightReqVO;
import com.huoran.iasf.vo.req.PaginationColumnReqVO;
import com.huoran.iasf.vo.resp.SortColumnRespVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.RedisConnectionFailureException;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


/**
 * @描述：栏目基础信息控制类
 * @作者: Rong
 * @日期: 2022-08-01
 */
@RestController
@RequestMapping("/sysColumn")
@Api(value = "栏目基础信息:SysColumnController", tags = "栏目管理")
public class SysColumnController {

    @Autowired
    public SysColumnService service;

    @Autowired
    public SysContentService contentService;

    @Autowired
    public SysTemplateService templateService;

    @Autowired
    public SysTemplateStyleService styleService;

    @Autowired
    public SysColumnLongPageService sysColumnLongPageService;

    @Autowired
    public SysTemplateStyleConfigurationService templateStyleConfigurationService;

    @Autowired
    public SysFilesService sysFilesService;
    @Autowired
    public RedisTemplate redisTemplate;

    @PostMapping("/listWithTree")
    @ApiOperation(value = "栏目树结构", response = SysColumn.class)
    public R listWithTree(@RequestBody @Valid PaginationColumnReqVO sysColumn) {
        return R.success(service.listWithTree(sysColumn));
    }

    @PostMapping("/listWithTreeMenuVisible")
    @ApiOperation(value = "栏目树结构(前台可见，只展示试单可见的栏目)", response = SysColumn.class)
    public R listWithTreeMenuVisible(@RequestBody @Valid PaginationColumnReqVO sysColumn) {
        return R.success(service.listWithTreeMenuVisible(sysColumn));
    }


    @PostMapping("/findById")
    @ApiOperation(value = "查询详情", response = SysColumn.class)
    public R findById(@RequestParam("id") @ApiParam(value = "主键ID") Integer id) {
        SysColumn sysColumn = service.getById(id);
        if (sysColumn == null) {
            throw new NotFoundException(BaseResponseCode.DATA_DOES_NOT_EXIST);
        }
        SysTemplateStyle sysTemplateStyle = styleService.getById(sysColumn.getListStyleId());

       if (!ObjectUtil.isEmpty(sysTemplateStyle)){
           if (!ObjectUtil.isEmpty(sysTemplateStyle.getPath())) {
               sysColumn.setPath(sysTemplateStyle.getPath());
           }
       }

        return R.success(sysColumn);
    }


    @NoRepeatSubmit
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = SysColumn.class)
    public R save(@RequestBody @Valid SysColumn sysColumn) {
        QueryWrapper<SysColumn> queryWrapper = new QueryWrapper<SysColumn>();
        queryWrapper.eq("father_id", sysColumn.getFatherId());
        Integer count = service.count(queryWrapper);
        sysColumn.setSort(count + 1);
        boolean addState = service.save(sysColumn);
        return addState ? R.success(sysColumn.getId()) : R.fail("新增失败");
    }

    @PostMapping("/update")
    @ApiOperation(value = "修改", response = SysColumn.class)
    public R update(@RequestBody SysColumn sysColumn) {
        boolean updateState = service.updateById(sysColumn);
        return updateState ? R.success() : R.fail("编辑失败");
    }

    /**
     * 缓存用户已选择或操作过的字段。
     *
     * @param userId 用户唯一标识，必须为正整数，用于定位特定用户的缓存字段。
     * @param fields 以逗号分隔的字段列表字符串，表示用户操作或关注的字段。不能为空字符串。
     * @return 成功时返回成功消息，失败时返回错误描述。
     */
    @PostMapping("/cacheField")
    @ApiOperation(value = "缓存用户存过的字段")
    public R cacheField(@ApiParam(name = "userId", value = "用户id", required = true) @RequestParam Integer userId,
                        @ApiParam(name = "fields", required = true, value = "字段(逗号隔开)") @RequestParam String fields,
                        @ApiParam(name = "siteId", required = true, value = "站点id") @RequestParam Integer siteId
    ) {

        try {
            String key = "fieldCache_" +"siteId_" +siteId +"userId_"+ userId;
            redisTemplate.opsForValue().set(key, fields);
            return R.success("字段缓存成功");
        } catch (RedisConnectionFailureException e) {
            return R.fail("由于Redis连接问题，缓存字段失败");
        }
    }



    /**
     * 根据用户ID和站点ID获取缓存的字段列表。
     * 该接口通过组合用户ID和站点ID作为键，从Redis缓存中获取字段列表。如果缓存中不存在相应的数据，
     * 或者数据的类型不匹配，则返回相应的错误信息。如果数据存在且类型匹配，则返回缓存的字段列表。
     *
     * @param userId 用户ID，用于构造缓存键，标识特定用户的缓存数据。
     * @param siteId 站点ID，用于构造缓存键，标识特定站点的缓存数据。
     * @return 如果缓存中存在数据且数据类型为String，则返回缓存的字段列表；否则返回空或错误信息。
     */
    @GetMapping("/getCacheField")
    @ApiOperation(value = "获取用户已缓存的字段列表")
    public R getCacheField(@ApiParam(name = "userId", value = "用户id", required = true) @RequestParam Integer userId,
                           @ApiParam(name = "siteId", required = true, value = "站点id") @RequestParam Integer siteId) {
        try {
            // 组合键名，用于在Redis中查找缓存数据。
            String key = "fieldCache_" +"siteId_" +siteId +"userId_"+ userId;
            // 从Redis中获取缓存数据。
            Object cachedFieldsObj = redisTemplate.opsForValue().get(key);

            // 检查缓存数据是否存在。
            if (cachedFieldsObj == null) {
                return R.success();
            }

            // 检查缓存数据类型是否为String。
            if (!(cachedFieldsObj instanceof String)) {
                return R.fail("缓存数据类型不匹配，预期为String类型");
            }

            // 将缓存的字符串转换为字段列表。
            // 直接将字符串分割成列表
            String cachedFieldsStr = (String) cachedFieldsObj;
            if (cachedFieldsStr.isEmpty()){
                return  R.success();
            }
            List<String> fields = Arrays.asList(cachedFieldsStr.split(","));


            return R.success(fields); // 直接返回分割后的字段列表
        } catch (RedisConnectionFailureException e) {
            // 如果Redis连接失败，返回错误信息。
            return R.fail("由于Redis连接问题，获取字段失败");
        }
    }


    @PostMapping("/controlDisplayNavigationMenu")
    @ApiOperation(value = "控制显示导航菜单", response = SysColumn.class)
    public R controlDisplayNavigationMenu(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id, @ApiParam(name = "menuVisible", value = "菜单是否可见(默认0可见 1不可见)", required = true) @RequestParam Integer menuVisible) {
        SysColumn column = new SysColumn();
        column.setId(id);
        column.setMenuVisible(menuVisible);//菜单是否可见(默认0可见 1不可见)

        boolean updateState = service.updateById(column);
        return updateState ? R.success() : R.fail("编辑失败");
    }



       /**
     * 批量删除栏目及其关联的内容和长页信息。
     * 接收一个包含栏目ID的列表，遍历这些ID以执行以下操作：
     * 1. 更新与栏目关联的文件记录为已删除状态。
     * 2. 递归收集每个栏目的子栏目ID至一个列表中，并包含当前栏目ID。
     * 3. 使用收集到的ID列表，删除关联的文章记录。
     * 4. 删除关联的长页栏目记录。
     * 5. 最后，根据ID列表删除栏目本身。
     *
     * @param ids 主键列表，标识需要删除的栏目ID。
     * @return 成功响应，无论实际删除操作是否成功均返回成功消息。
     */
    @NoRepeatSubmit
    @PostMapping("/batchDeletion")
    @ApiOperation(value = "批量删除栏目", response = SysColumn.class)
    public R batchDeletion(@ApiParam(name = "ids", value = "栏目ID列表", required = true) @RequestParam List<Integer> ids) {
        for (Integer id : ids) {
            // 更新与栏目相关的文件记录为已删除
            UpdateWrapper<SysFilesEntity> filesUpdateWrapper = new UpdateWrapper<>();
            filesUpdateWrapper.eq("quote_id", id).eq("quote_type", 1).set("is_del", 1);
            sysFilesService.update(filesUpdateWrapper);

            // 收集当前栏目及其子栏目的ID
            List<Integer> idList = new ArrayList<>();
            this.selectCategoryChildById(id.toString(), idList);
            // 添加当前栏目ID到待处理列表
            idList.add(id);

            // 删除栏目下所有文章
            QueryWrapper<SysContent> contentQueryWrapper = new QueryWrapper<>();
            contentQueryWrapper.in("column_id", idList);
            contentService.remove(contentQueryWrapper);

            // 删除关联的长页栏目信息
            QueryWrapper<SysColumnLongPage> longPageQueryWrapper = new QueryWrapper<>();
            longPageQueryWrapper.in("column_id", idList);
            sysColumnLongPageService.remove(longPageQueryWrapper);

            // 删除栏目
            service.removeByIds(idList);

        }
        return R.success();
    }



    @PostMapping("/sameLevelJudgment")
    @ApiOperation(value = "同级判重", response = ColumnWeightReqVO.class)
    public R sameLevelJudgment(@RequestBody @Valid ColumnWeightReqVO sysColumn) {
        return service.sameLevelJudgment(sysColumn) ? R.success() : R.fail("同级下已存在重复栏目!");
    }

        /**
     * 根据传入的栏目排序信息更新栏目顺序。
     * 遍历接收到的排序信息列表，针对每个栏目验证同级下是否有名称相同的栏目存在（排除自身），
     * 若无冲突则更新该栏目的排序信息。
     *
     * @param sortColumnRespVOList 包含栏目ID、父栏目ID等排序信息的列表。
     * @return 如果更新成功或无冲突，则返回操作成功的响应；若检测到同级下存在名称重复的栏目，则返回失败信息。
     */
    @PostMapping("/sortByColumn")
    @ApiOperation(value = "栏目排序(依据id更改)", response = SysColumn.class)
    public R sortByColumn(@RequestBody List<SortColumnRespVO> sortColumnRespVOList) {
        for (SortColumnRespVO sortInfo : sortColumnRespVOList) {
            // 获取当前要排序的栏目信息
            SysColumn existingColumn = service.getById(sortInfo.getId());

            // 构建查询条件，检查同级下是否有名称相同的其他栏目（排除自己）
            QueryWrapper<SysColumn> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("column_name", existingColumn.getColumnName())
                        .eq("site_id", existingColumn.getSiteId())
                        .eq("father_id", sortInfo.getFatherId())
                        .ne("id", sortInfo.getId()); // 确保不包括当前正在处理的栏目

            // 执行查询
            List<SysColumn> conflictingColumns = service.list(queryWrapper);

            // 检查是否有冲突
            if (!conflictingColumns.isEmpty()) {
                return R.fail("同级下已存在重复栏目:" + existingColumn.getColumnName());
            }

            // 无冲突，复制排序信息并更新数据库
            SysColumn updateColumn = new SysColumn();
            BeanUtils.copyProperties(sortInfo, updateColumn);
            service.updateById(updateColumn);
        }

        // 全部处理完毕，返回成功响应
        return R.success();
    }


    @ApiOperation("获取一级下的子级栏目")
    @PostMapping("/getsTheSubColumn")
    public R getsTheSubColumn(@ApiParam(name = "id", value = "id", required = true) @RequestParam Integer id, @ApiParam(name = "siteId", value = "站点id", required = true) @RequestParam Integer siteId) {
        String ids = this.getIds(id);
        return service.getsTheSubColumn(id, ids, siteId);

    }


    @ApiOperation("获取某层级下的子级栏目")
    @PostMapping("/getsSublevelColumnsUnderALevel")
    public R getsSublevelColumnsUnderALevel(@ApiParam(name = "id", value = "id", required = true) @RequestParam Integer id, @ApiParam(name = "siteId", value = "站点id", required = true) @RequestParam Integer siteId) {
        String ids = this.getIds(id);
        return service.getsTheSubColumn(id, ids, siteId);

    }


    public String getIds(Integer id) {
        //1 创建list集合，用于封装所有删除菜单id值
        List<Integer> idList = new ArrayList<>();
        //2 向idList集合设置删除菜单id
        this.selectCategoryChildById(id + "", idList);//找到当前菜单的子菜单  把结果id封装到idlist里面去
        //把当前id封装到list里面
        idList.add(id);//现在把当前id封装进去  之前都是子菜单的id
        String str = "";
        for (Integer idstr : idList) {
            str += idstr + ",";
        }
        String ids = str.substring(0, str.length() - 1);
        return ids;
    }

    public void selectCategoryChildById(String id, List<Integer> idList) {
        //查询菜单里面子菜单id
        QueryWrapper<SysColumn> wrapper = new QueryWrapper<SysColumn>();
        wrapper.eq("father_id", id);
        wrapper.eq("deleted", Constant.DATA_NOT_DELETED);
        //查询指定列的值
        wrapper.select("id");
        //当前菜单的所有子菜单
        List<SysColumn> childIdList = service.list(wrapper);
        //把childIdList里面菜单id值获取出来，封装idList里面，做递归查询
        //遍历集合得到每一个对象item
        childIdList.stream().forEach(item -> {
            //封装idList里面
            idList.add(item.getId());
            //递归查询
            this.selectCategoryChildById(item.getId() + "", idList);
        });
    }


    @ApiOperation("获取一级下的子级栏目以及下的全部文章标签")
    @PostMapping("/getTheFullArticleByColumn")
    public R getTheFullArticleByColumn(@ApiParam(name = "id", value = "一级栏目id", required = true) @RequestParam Integer id) {
        String ids = this.getIds(id);
        return contentService.getTheFullArticleByColumn(ids);
    }


    @ApiOperation("获取子级栏目id")
    @PostMapping("/getsTheChildColumnId")
    public R getsTheSublevelColumnIdOfTheNextLevel(@ApiParam(name = "id", value = "一级栏目id", required = true) @RequestParam Integer id) {
        //1 创建list集合，用于封装所有删除菜单id值
        List<Integer> idList = new ArrayList<>();
        //2 向idList集合设置删除菜单id
        this.selectCategoryChildById(id + "", idList);//找到当前菜单的子菜单  把结果id封装到idlist里面去
        //把当前id封装到list里面
        idList.add(id);//现在把当前id封装进去  之前都是子菜单的id
        return R.success(idList);

    }

    @ApiOperation("某一级查全部")
    @PostMapping("/oneLevelChecksThemAll")
    public R oneLevelChecksThemAll(@ApiParam(name = "id", value = "栏目id", required = true) @RequestParam Integer id, @ApiParam(name = "isSort", value = "判断是否为排序接口调用(1为排序接口调用 0我栏目管理列表调用)", required = true) @RequestParam Integer isSort, @ApiParam(name = "ids", value = "主键", required = true) @RequestParam Integer siteId) {

        Integer pid = service.oneLevelChecksThemAll(id);
        String ids = this.getIds(pid);
        return service.getSubColumn(pid, ids, isSort, siteId);
    }


    /**
     * 查询栏目下的文章(若选中栏目与子级全部栏目类型一致返回全部文章，反之查询选中栏目下的文章)
     * 2023.02.22依需求更改为以发布时间排序(非创建时间)
     *
     * @param columnId
     * @return
     */
    @ApiOperation("查询栏目下的文章(若选中栏目与子级全部栏目类型一致返回全部文章，反之查询选中栏目下的文章)")
    @PostMapping("/queryArticlesByColumnType")
    public R queryArticlesByColumnType(@ApiParam(name = "columnId", value = "栏目id", required = true) @RequestParam Integer columnId) {
        String ids = this.getIds(columnId);
        return service.queryArticlesByColumnType(ids, columnId);
    }


    /**
     * 删除标记为无用的数据，包括相关联的文件引用、文章、以及长页栏目条目。
     * 此方法首先查询所有未被标记为删除的栏目，然后对每个栏目执行以下操作：
     * 1. 更新与栏目关联的文件记录为已删除状态。
     * 2. 收集当前栏目及其所有子栏目的ID。
     * 3. 使用收集到的ID列表，删除关联的文章记录和长页栏目记录。
     * 4. 最后，删除这些栏目本身。
     *
     * @return 成功响应，表示删除无用数据的操作已完成。
     */
    @NoRepeatSubmit
    @PostMapping("/deleteUselessData")
    @ApiOperation(value = "删除无用数据", response = SysColumn.class)
    public R deleteUselessData() {
        // 查询未被标记为删除的栏目ID
        QueryWrapper<SysColumn> columnQueryWrapper = new QueryWrapper<>();
        columnQueryWrapper.eq("deleted", 0).select("id");
        List<SysColumn> columnsToProcess = service.list(columnQueryWrapper);

        for (SysColumn column : columnsToProcess) {
            // 标记与栏目关联的文件为已删除
            UpdateWrapper<SysFilesEntity> fileUpdateWrapper = new UpdateWrapper<>();
            fileUpdateWrapper.eq("quote_id", column.getId()).eq("quote_type", 1).set("is_del", 1);
            sysFilesService.update(fileUpdateWrapper);

            // 收集当前栏目及其子栏目ID
            List<Integer> relatedIds = new ArrayList<>();
            this.selectCategoryChildById(column.getId().toString(), relatedIds);
            relatedIds.add(column.getId()); // 包含当前栏目ID

            // 删除关联的文章
            QueryWrapper<SysContent> contentQueryWrapper = new QueryWrapper<>();
            contentQueryWrapper.in("column_id", relatedIds);
            contentService.remove(contentQueryWrapper);

            // 删除关联的长页栏目
            QueryWrapper<SysColumnLongPage> longPageQueryWrapper = new QueryWrapper<>();
            longPageQueryWrapper.in("column_id", relatedIds);
            sysColumnLongPageService.remove(longPageQueryWrapper);

            // 删除栏目
            service.removeByIds(relatedIds);
        }

        // 全部处理完成后返回成功响应
        return R.success();
    }

}

