package com.huoran.iasf.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huoran.iasf.common.exception.NotFoundException;
import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.entity.SysColumn;
import com.huoran.iasf.entity.SysContent;
import com.huoran.iasf.entity.SysContentFile;
import com.huoran.iasf.entity.SysFilesEntity;
import com.huoran.iasf.service.SysColumnService;
import com.huoran.iasf.service.SysContentFileService;
import com.huoran.iasf.service.SysContentService;
import com.huoran.iasf.service.SysFilesService;
import com.huoran.iasf.vo.req.ContentHeavyTitleReqVO;
import com.huoran.iasf.vo.req.ContentReq;
import com.huoran.iasf.vo.req.PageContentReqVO;
import com.huoran.iasf.vo.resp.PageContentRespVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.RedisConnectionFailureException;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.Date;
import java.util.List;


/**
 * @描述：文章管理控制类
 * @作者: Rong
 * @日期: 2022-08-05
 */
@RestController
@RequestMapping("/sysContent")
@Api(value = "文章管理:SysContentController", tags = "文章管理")
public class SysContentController {

    @Autowired
    public SysContentService service;
    @Autowired
    public SysColumnService columnService;

    @Autowired
    public SysContentFileService fileService;

    @Autowired
    public SysFilesService sysFilesService;
    @Autowired
    public RedisTemplate redisTemplate;

    @PostMapping("/pagingQuery")
    @ApiOperation(value = "分页查询文章列表", response = PageContentRespVO.class)
    public R pagingQuery(@RequestBody @Valid @ApiParam(name = "分页查询参数", value = "传入json格式", required = true) PageContentReqVO sysContent) {
        return service.articlePaginationList(sysContent);
    }


    @PostMapping("/findById")
    @ApiOperation(value = "查询详情", response = SysContent.class)
    public R findById(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id) {


        SysContent sysContent = service.getById(id);
        if (sysContent == null) {
            throw new NotFoundException(BaseResponseCode.DATA_DOES_NOT_EXIST);
        }

        SysColumn sysColumn = columnService.getById(sysContent.getColumnId());
        if (sysColumn == null) {
            throw new NotFoundException(BaseResponseCode.DATA_DOES_NOT_EXIST);
        }
        /*if (sysColumn == null) {
            throw new NotFoundException(BaseResponseCode.DATA_DOES_NOT_EXIST);
        }*/
        if (sysColumn.getColumnName() != null) {
            sysContent.setColumnName(sysColumn.getColumnName());
        }
        List<SysContentFile> fileList = fileService.getFileByContentId(id);
        sysContent.setFileList(fileList);


        return R.success(sysContent);
    }

    @PostMapping("/save")
    @ApiOperation(value = "新增", response = SysContent.class)
    public R save(@RequestBody @Valid @ApiParam(name = "文章管理对象", value = "传入json格式", required = true) SysContent sysContent) {

      /*  QueryWrapper<SysContent> queryWrap = new QueryWrapper<>();
        queryWrap.eq("column_id", sysContent.getColumnId());
        queryWrap.eq("deleted", 1);
        Integer count = service.count(queryWrap);
        if (count == 0) {
            count = 1;
        }
        sysContent.setSequence(count + 1);*/
        boolean addState = service.save(sysContent);


        return addState ? R.success(sysContent.getId()) : R.fail("新增失败");
    }


    @PostMapping("/update")
    @ApiOperation(value = "修改", response = SysContent.class)
    public R update(@RequestBody @ApiParam(name = "文章管理对象", value = "传入json格式", required = true) SysContent sysContent) {
        boolean updateState = service.updateById(sysContent);
        return updateState ? R.success() : R.fail("编辑失败");
    }


    @PostMapping("/delete")
    @ApiOperation(value = "删除", response = SysContent.class)
    public R delete(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id) {
        UpdateWrapper<SysFilesEntity> queryWrapper = new UpdateWrapper<>();
        queryWrapper.eq("quote_id", id);
        queryWrapper.eq("quote_type", 0);
        queryWrapper.set("is_del", 1);
        sysFilesService.update(queryWrapper);
        boolean delState = service.removeById(id);
        return delState ? R.success() : R.fail("删除失败");
    }


    @PostMapping("/batchDeletion")
    @ApiOperation(value = "批量删除文章", response = SysContent.class)
    public R batchDeletion(@ApiParam(name = "ids", value = "主键", required = true) @RequestParam List<Integer> ids) {
        for (Integer id : ids) {
            UpdateWrapper<SysFilesEntity> queryWrapper = new UpdateWrapper<>();
            queryWrapper.eq("quote_id", id);
            queryWrapper.eq("quote_type", 0);
            queryWrapper.set("is_del", 1);
            sysFilesService.update(queryWrapper);
        }
        boolean delState = service.removeByIds(ids);
        return delState ? R.success() : R.fail("删除失败");
    }

    @PostMapping("/checkIfTheTitleIsRepeat")
    @ApiOperation(value = "标题判重——只对已发布的判重(新增，编辑，点击发布时候都要调用判断))", response = ContentHeavyTitleReqVO.class)
    public R checkIfTheTitleIsRepeat(@RequestBody @Valid ContentHeavyTitleReqVO content) {
        return service.checkIfTheTitleIsRepeat(content) ? R.success() : R.fail("该标题已重复!");
    }

    @PostMapping("/newlyPublishedArticles")
    @ApiOperation(value = "站点最新发布的文章", response = PageContentReqVO.class)
    public R newlyPublishedArticles(@Valid @RequestBody PageContentReqVO content) {
        return service.newlyPublishedArticles(content);
    }


    /*查询已发布的、启用了的文章列表接口*/
/*    @PostMapping("/publishedEnabledArticles")
    @ApiOperation(value = "查询已发布的、启用了的文章列表", response = PageContentReqVO.class)
    public R publishedEnabledArticles() {
        return service.publishedEnabledArticles();
    }*/


    /**
     * 获取栏目下的文章
     *
     * @param pageReq
     * @return
     */
    @PostMapping("/getColumnArticles")
    @ApiOperation(value = "获取栏目下的文章", response = SysContent.class)
    public R getColumnArticles(@RequestBody @ApiParam(name = "文章管理对象", value = "传入json格式", required = true) ContentReq pageReq) {
        IPage page = new Page(pageReq.getPageNum(), pageReq.getPageSize());
        QueryWrapper<SysContent> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("column_id", pageReq.getColumnIds());
        IPage<SysContent> iPage = service.page(page, queryWrapper);
        return R.success(iPage);
    }


    @PostMapping("/articleEnableOrDisable")
    @ApiOperation(value = "文章启用禁用", response = SysContent.class)
    public R articleEnableOrDisable(@ApiParam(name = "id", value = "文章id", required = true) @RequestParam Integer id, @ApiParam(name = "isDisable", value = "是否禁用(0默认，0启用 1禁用)", required = true) @RequestParam Integer isDisable) {
        UpdateWrapper<SysContent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("is_disable", isDisable);
        updateWrapper.eq("id", id);
        boolean ret = service.update(updateWrapper);
        return ret ? R.success() : R.fail("禁用/启用失败");
    }

    @PostMapping("/articlePreview")
    @ApiOperation(value = "增加文章浏览量(点击一次算一次)", response = SysContent.class)
    public R articlePreview(@ApiParam(name = "contentId", value = "文章id", required = true) @RequestParam Integer contentId) {
        return R.success(service.statisticsOfPageViews(contentId));

    }


    @PostMapping("/hotContent")
    @ApiOperation(value = "热点内容", response = SysContent.class)
    public R hotContent(@ApiParam(name = "siteId", value = "站点id", required = true) @RequestParam Integer siteId) {
        return R.success(service.hotContent(siteId));

    }

    @PostMapping("/siteSearchArticles")
    @ApiOperation(value = "站点搜索文章", response = SysContent.class)
    public R siteSearchArticles(@RequestBody PageContentReqVO req) {
        return service.siteSearchArticles(req);

    }


    @PostMapping("/articleTopOperation")
    @ApiOperation(value = "文章置顶/取消置顶", response = SysContent.class)
    public R articleTopOperation(@ApiParam(name = "isTop", value = "是否置顶(默认为0 不置顶 1为置顶)", required = true) @RequestParam Integer isTop, @ApiParam(name = "articleId", value = "文章Id", required = true) @RequestParam Integer articleId) {
        //是否置顶(默认为0 不置顶 1为置顶)
        UpdateWrapper<SysContent> updateWrap = new UpdateWrapper<>();

        if (isTop == 1) {
            //置顶时间(置顶一次更新一次)
            updateWrap.set("top_time", new Date());
        } else {
            updateWrap.set("top_time", null);
        }
        updateWrap.set("is_top", isTop);
        updateWrap.eq("id", articleId);
        updateWrap.eq("deleted", 1);
        boolean ret = service.update(updateWrap);
        return ret ? R.success() : R.fail("操作失败");

    }


    @PostMapping("/queryArticleColumnParent")
    @ApiOperation(value = "查询文章的栏目父级", response = PageContentReqVO.class)
    public R queryArticleColumnParent(@ApiParam(name = "articleId", value = "文章id", required = true) @RequestParam Integer articleId) {
        return service.queryArticleColumnParent(articleId);
    }

    @PostMapping("/cacheField")
    @ApiOperation(value = "缓存用户存过的字段")
    public R cacheField(@ApiParam(name = "userId", value = "用户id", required = true) @RequestParam Integer userId,
                        @ApiParam(name = "fields", required = true, value = "字段(逗号隔开)") @RequestParam String fields,
                        @ApiParam(name = "siteId", required = true, value = "站点id") @RequestParam Integer siteId
    ) {

        try {
            String key = "articleFieldCache_" + "siteId_" + siteId + "userId_" + userId;
            redisTemplate.opsForValue().set(key, fields);
            return R.success("字段缓存成功");
        } catch (RedisConnectionFailureException e) {
            return R.fail("由于Redis连接问题，缓存字段失败");
        }
    }


    /**
     * 获取用户已缓存的字段列表。
     * 将Redis中存储的字符串（以逗号分隔的字段）直接转换为字符串列表并返回。
     *
     * @param userId 用户唯一标识，用于查询该用户在Redis中的缓存字段。
     * @return 如果用户有缓存的字段，返回一个以逗号分隔的字段字符串（不包含任何额外符号）；
     * 如果用户没有缓存的字段，则返回特定的消息提示；
     * 遇到Redis连接问题时返回错误信息。
     */
    @GetMapping("/getCacheField")
    @ApiOperation(value = "获取用户已缓存的字段列表")
    public R getCacheField(@ApiParam(name = "userId", value = "用户id", required = true) @RequestParam Integer userId,
                           @ApiParam(name = "siteId", required = true, value = "站点id") @RequestParam Integer siteId) {
        try {
            String key = "articleFieldCache_" + "siteId_" + siteId + "userId_" + userId;
            Object cachedFieldsObj = redisTemplate.opsForValue().get(key);

            if (cachedFieldsObj == null) {
                return R.success();
            }

            if (!(cachedFieldsObj instanceof String)) {
                return R.fail("缓存数据类型不匹配，预期为String类型");
            }

            // 直接将字符串分割成列表
            String cachedFieldsStr = (String) cachedFieldsObj;
            if (cachedFieldsStr.isEmpty()){
                return R.success();
            }
            List<String> fields = Arrays.asList(cachedFieldsStr.split(","));

            return R.success(fields); // 直接返回分割后的字段列表
        } catch (RedisConnectionFailureException e) {
            return R.fail("由于Redis连接问题，获取字段失败");
        }
    }

}

