package com.huoran.iasf.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.entity.SysFloatingColumnScheme;
import com.huoran.iasf.entity.SysFloatingColumnSchemeModule;
import com.huoran.iasf.entity.SysFloatingColumnSchemeScopeOfApplication;
import com.huoran.iasf.service.SysFloatingColumnSchemeModuleService;
import com.huoran.iasf.service.SysFloatingColumnSchemeScopeOfApplicationService;
import com.huoran.iasf.service.SysFloatingColumnSchemeService;
import com.huoran.iasf.vo.ConflictCheckResultVO;
import com.huoran.iasf.vo.SysFloatingColumnSchemeVO;
import com.huoran.iasf.vo.req.SuspensionBarListPagingReq;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 * @描述：悬浮栏方案控制类
 * @作者: Rong
 * @日期: 2024-06-04
 */
@RestController
@RequestMapping("/SysFloatingColumnSchemeService/SysFloatingColumnScheme")
@Api(value = "悬浮栏相关接口:SysFloatingColumnSchemeController", tags = "悬浮栏方案:悬浮栏相关接口")
public class SysFloatingColumnSchemeController {

    @Autowired
    public SysFloatingColumnSchemeService service;

    @Autowired
    public SysFloatingColumnSchemeScopeOfApplicationService floatingColumnSchemeScopeOfApplicationService;

    @Autowired
    public SysFloatingColumnSchemeModuleService floatingColumnSchemeModuleService;


    /**
     * 保存或更新悬浮栏方案记录
     * 如果ID为空，则执行保存操作；如果ID有值，则视为更新操作。
     *
     * @param sysFloatingColumnScheme 要保存或更新的悬浮栏方案对象
     * @return R 结果对象，包含操作状态信息
     */
    @PostMapping("/saveOrUpdate")
    @ApiOperation(value = "保存或更新悬浮栏方案记录", response = SysFloatingColumnScheme.class)
    public R saveOrUpdate(@RequestBody @ApiParam(name = "悬浮栏方案对象", value = "包含悬浮栏方案详细信息，若id不存在则新建，否则视为更新", required = true) SysFloatingColumnScheme sysFloatingColumnScheme) {

        // 校验集合内是否有重复的schemeTitle
        Set<String> schemeTitles = new HashSet<>();
        for (SysFloatingColumnSchemeModule module : sysFloatingColumnScheme.getFloatingColumnSchemeModules()) {
            if (!schemeTitles.add(module.getSchemeTitle())) {
//                throw new IllegalArgumentException(module.getSchemeTitle() + " 标题已存在");

                String msg = module.getSchemeTitle() + " 标题已存在";

                /*throw new RuntimeException(msg);*/
                return R.fail(msg);
            }
        }

        if (sysFloatingColumnScheme.getId() != null&&sysFloatingColumnScheme.getIsDisable()==0){
            ConflictCheckResultVO conflictCheckResultVO = service.preCheckMultiScopeConflictBeforeUpdate(sysFloatingColumnScheme.getSiteId(),
                    sysFloatingColumnScheme.getFloatingColumnSchemeScopeOfApplications(),sysFloatingColumnScheme.getId());
            if (conflictCheckResultVO.isHasConflict()) {
                return R.fail(conflictCheckResultVO.getMessage());
            }
        }


        // 新增场景下直接进行校验
        if (sysFloatingColumnScheme.getId() == null) {
            if (checkSchemeName(sysFloatingColumnScheme.getSchemeName(), sysFloatingColumnScheme.getSiteId())) {
                return R.fail("该站点下已存在相同的悬浮栏方案名称");
            }
        }
        // 编辑场景下，只有当名称发生改变时才进行校验，并排除当前记录自身
        else if (!sysFloatingColumnScheme.getSchemeName().equals(service.getById(sysFloatingColumnScheme.getId()).getSchemeName())) {
            if (checkSchemeNameExceptSelf(sysFloatingColumnScheme.getSchemeName(), sysFloatingColumnScheme.getSiteId(), sysFloatingColumnScheme.getId())) {
                return R.fail("该站点下已存在相同的悬浮栏方案名称");
            }
        }


        //获取floatingColumnSchemeScopeOfApplications中的范围判断是不是0,如果包含为0
        List<SysFloatingColumnSchemeScopeOfApplication> floatingColumnSchemeScopeOfApplications = sysFloatingColumnScheme.getFloatingColumnSchemeScopeOfApplications();
        for (SysFloatingColumnSchemeScopeOfApplication floatingColumnSchemeScopeOfApplication : floatingColumnSchemeScopeOfApplications) {
            if (floatingColumnSchemeScopeOfApplication.getApplicationScopeId().equals("0")) {
                sysFloatingColumnScheme.setIsGlobal(1);

            } else {
                sysFloatingColumnScheme.setIsGlobal(0);
            }
        }


        // 处理全局悬浮栏逻辑
        if (sysFloatingColumnScheme.getIsGlobal() == 1) {
            // 确保没有其他全局悬浮栏同时启用
            long globalCount = service.count(new QueryWrapper<SysFloatingColumnScheme>().eq("is_global", true)
                    .eq("site_id", sysFloatingColumnScheme.getSiteId())
                    .eq("is_disable", 0).ne("id", sysFloatingColumnScheme.getId()));
            if (globalCount > 0) {
                return R.fail("已经有全局悬浮栏启用，请先禁用后再尝试设置");
            }
        }

        boolean result = false;

        try {
            if (sysFloatingColumnScheme.getId() == null) {
                // 保存操作
                result = service.save(sysFloatingColumnScheme);
            } else {
                // 更新操作
                result = service.updateById(sysFloatingColumnScheme);
            }
        } catch (Exception e) {
            return R.fail("操作失败");
        }

        if (!result) {
            return R.fail("操作失败");
        }

        //删除原有的方案和范围关系

        floatingColumnSchemeModuleService.deleteByFloatingBarSchemeId(sysFloatingColumnScheme.getId());
        floatingColumnSchemeScopeOfApplicationService.deleteByFloatingBarSchemeId(sysFloatingColumnScheme.getId());


        // 保持原有的模块和范围处理逻辑
        handleFloatingColumnSchemeModules(sysFloatingColumnScheme);
        handleFloatingColumnSchemeScopeOfApplications(sysFloatingColumnScheme);

        return R.success();
    }


    @PostMapping("/columnDisplayFloatingBar")
    @ApiOperation(value = "根据栏目展示悬浮栏", response = SysFloatingColumnSchemeVO.class)
    public R columnDisplayFloatingBar(@ApiParam(name = "columnId", value = "栏目id", required = true) @RequestParam String columnId) {
        return service.columnDisplayFloatingBar(columnId);
    }


    @PostMapping("/floatingBarList")
    @ApiOperation(value = "悬浮栏列表", response = SysFloatingColumnSchemeVO.class)
    public R floatingBarList(@RequestBody SuspensionBarListPagingReq sysFloatingColumnScheme) {
        return service.floatingBarList(sysFloatingColumnScheme);
    }


    /**
     * * 1.应用全部权限最高，只要是启用应用全部的，除了一开始没有启用悬浮栏的都弹窗提示，当前已启用了的悬浮栏方案将会被取消。
     * 2.启用了应用全部的，后面要启用部分的，提示先禁用应用全部的。
     * 3.部分启用vs部分启用，且有重合的，提示移除重合的。
     */
    @PostMapping("/enableOrDisableScheme")
    @ApiOperation(value = "启用或禁用", response = SysFloatingColumnSchemeVO.class)
    public R enableOrDisableScheme(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id, @ApiParam(name = "isDisable", value = "是否禁用(0启用 1禁用)", required = true) @RequestParam Integer isDisable) {
        /* return service.enableOrDisableScheme(id, isDisable);*/

        //禁启用操作
        SysFloatingColumnScheme sysFloatingColumnScheme = service.getById(id);
        sysFloatingColumnScheme.setIsDisable(isDisable);
        boolean operation = service.updateById(sysFloatingColumnScheme);
        //增加判断如果当前的应用权限如果为全部，需要将当前站点下其余已经启用的给禁用了
        if (sysFloatingColumnScheme.getIsDisable() == 0 && sysFloatingColumnScheme.getIsGlobal() == 1) {
            UpdateWrapper<SysFloatingColumnScheme> updateWrapper = new UpdateWrapper<>();
            updateWrapper.set("is_disable", 1);
            updateWrapper.eq("is_disable", 0);
            updateWrapper.eq("site_id", sysFloatingColumnScheme.getSiteId());
            updateWrapper.ne("id", id);
            service.update(updateWrapper);

        }


        return operation ? R.success() : R.fail("操作失败");
    }


    @PostMapping("/checkEnableOrDisable")
    @ApiOperation(value = "检查“启用”或“禁用”()", response = SysFloatingColumnSchemeVO.class)
    public R checkEnableOrDisable(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id, @ApiParam(name = "isDisable", value = "是否禁用(0启用 1禁用)", required = true) @RequestParam Integer isDisable) {

        return service.checkEnableOrDisable(id, isDisable);
    }


    @PostMapping("/findById")
    @ApiOperation(value = "查询详情", response = SysFloatingColumnScheme.class)
    public R findById(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id) {
        SysFloatingColumnScheme sysFloatingColumnScheme = service.getById(id);
        List<SysFloatingColumnSchemeScopeOfApplication> sysFloatingColumnSchemeScopeOfApplications = floatingColumnSchemeScopeOfApplicationService.getScopeOfApplicationByFloatingBarSchemeId(id);
        List<SysFloatingColumnSchemeModule> getModuleByFloatingBarSchemeId = floatingColumnSchemeModuleService.getModuleByFloatingBarSchemeId(id);
        sysFloatingColumnScheme.setFloatingColumnSchemeScopeOfApplications(sysFloatingColumnSchemeScopeOfApplications);
        sysFloatingColumnScheme.setFloatingColumnSchemeModules(getModuleByFloatingBarSchemeId);
        return R.success(sysFloatingColumnScheme);
    }


    @PostMapping("/batchDeletion")
    @ApiOperation(value = "批量删除悬浮栏数据", response = SysFloatingColumnScheme.class)
    public R batchDeletion(@ApiParam(name = "id", value = "主键", required = true) @RequestParam List<Integer> id) {
        boolean delState = service.removeByIds(id);

        //删除的同时将悬浮栏方案以及悬浮栏应用范围
        if (delState) {
            for (Integer removeId : id) {


                LambdaQueryWrapper<SysFloatingColumnSchemeModule> queryWrapper = new LambdaQueryWrapper<>();
                queryWrapper.eq(SysFloatingColumnSchemeModule::getFloatingBarSchemeId, removeId);
                floatingColumnSchemeModuleService.remove(queryWrapper);

                LambdaQueryWrapper<SysFloatingColumnSchemeScopeOfApplication> queryWrapper1 = new LambdaQueryWrapper<>();
                queryWrapper1.eq(SysFloatingColumnSchemeScopeOfApplication::getFloatingBarSchemeId, removeId);
                floatingColumnSchemeScopeOfApplicationService.remove(queryWrapper1);
            }


        }

        return R.success();
    }


    @PostMapping("/batchDeletionScheme")
    @ApiOperation(value = "批量删除方案", response = SysFloatingColumnSchemeModule.class)
    public R batchDeletionScheme(@ApiParam(name = "id", value = "主键", required = true) @RequestParam List<Integer> id) {
        boolean delState = floatingColumnSchemeModuleService.removeByIds(id);
        return delState ? R.success() : R.fail("删除失败");
    }


    @PostMapping("/batchDeleteRange")
    @ApiOperation(value = "批量删除应用范围", response = SysFloatingColumnSchemeScopeOfApplication.class)
    public R batchDeleteRange(@ApiParam(name = "id", value = "主键", required = true) @RequestParam List<Integer> id) {
        boolean delState = floatingColumnSchemeScopeOfApplicationService.removeByIds(id);
        return delState ? R.success() : R.fail("删除失败");
    }


    private void handleFloatingColumnSchemeModules(SysFloatingColumnScheme sysFloatingColumnScheme) {
        /*// 校验集合内是否有重复的schemeTitle
        Set<String> schemeTitles = new HashSet<>();
        for (SysFloatingColumnSchemeModule module : sysFloatingColumnScheme.getFloatingColumnSchemeModules()) {
            if (!schemeTitles.add(module.getSchemeTitle())) {
//                throw new IllegalArgumentException(module.getSchemeTitle() + " 标题已存在");

                String  msg = module.getSchemeTitle() + " 标题已存在";

                throw new RuntimeException(msg);
            }
        }*/
        if (sysFloatingColumnScheme.getFloatingColumnSchemeModules() != null && !sysFloatingColumnScheme.getFloatingColumnSchemeModules().isEmpty()) {
            for (SysFloatingColumnSchemeModule module : sysFloatingColumnScheme.getFloatingColumnSchemeModules()) {
                module.setFloatingBarSchemeId(sysFloatingColumnScheme.getId()); // 确保模块关联到当前方案
                if (sysFloatingColumnScheme.getId() == null) {
                    // 如果悬浮栏方案是新建的，直接保存模块（假设模块没有独立的ID生成策略）
                    floatingColumnSchemeModuleService.save(module);
                } else {
                    // 如果悬浮栏方案是更新的，根据情况更新或保存模块
                    floatingColumnSchemeModuleService.saveOrUpdate(module);
                }
            }
        }
    }

    private void handleFloatingColumnSchemeScopeOfApplications(SysFloatingColumnScheme sysFloatingColumnScheme) {
        if (sysFloatingColumnScheme.getFloatingColumnSchemeScopeOfApplications() != null && !sysFloatingColumnScheme.getFloatingColumnSchemeScopeOfApplications().isEmpty()) {
            for (SysFloatingColumnSchemeScopeOfApplication scope : sysFloatingColumnScheme.getFloatingColumnSchemeScopeOfApplications()) {
                scope.setFloatingBarSchemeId(sysFloatingColumnScheme.getId());
                if (sysFloatingColumnScheme.getId() == null) {
                    // 如果悬浮栏方案是新建的，直接保存范围
                    floatingColumnSchemeScopeOfApplicationService.save(scope);
                } else {
                    // 如果悬浮栏方案是更新的，根据情况更新或保存范围
                    floatingColumnSchemeScopeOfApplicationService.saveOrUpdate(scope);
                }
            }
        }
    }


    // 根据站点校验方案名称，排除当前ID记录
    private boolean checkSchemeNameExceptSelf(String schemeName, Integer siteId, Integer id) {
        QueryWrapper<SysFloatingColumnScheme> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("scheme_name", schemeName);
        queryWrapper.eq("site_id", siteId);
        // 添加条件，排除当前ID的记录
        if (id != null) {
            queryWrapper.ne("id", id);
        }
        return service.count(queryWrapper) > 0;
    }


    // 现在仅用于新增场景或编辑时名称变更的校验，不需排除自身ID
    private boolean checkSchemeName(String schemeName, Integer siteId) {
        QueryWrapper<SysFloatingColumnScheme> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("scheme_name", schemeName);
        queryWrapper.eq("site_id", siteId);
        return service.count(queryWrapper) > 0;
    }
}

