package com.huoran.iasf.controller;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.entity.SysFloatingColumnScheme;
import com.huoran.iasf.entity.SysFooterSetup;
import com.huoran.iasf.entity.SysFooterSetupScopeOfApplication;
import com.huoran.iasf.service.SysFooterSetupScopeOfApplicationService;
import com.huoran.iasf.service.SysFooterSetupService;
import com.huoran.iasf.vo.ConflictCheckResultVO;
import com.huoran.iasf.vo.FooterListVO;
import com.huoran.iasf.vo.SysFloatingColumnSchemeVO;
import com.huoran.iasf.vo.req.SuspensionBarListPagingReq;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @描述：页脚配置控制类
 * @作者: Rong
 * @日期: 2024-06-07
 */
@RestController
@RequestMapping("/SysFooterSetupService/SysFooterSetup")
@Api(value = "页脚配置:SysFooterSetupController", tags = "页脚配置:页脚配置")
public class SysFooterSetupController {

    @Autowired
    public SysFooterSetupService service;

    @Autowired
    public SysFooterSetupScopeOfApplicationService footerSetupScopeOfApplicationService;


    // 根据站点校验方案名称，排除当前ID记录
    private boolean checkSchemeNameExceptSelf(String footerName, Integer siteId, Integer id) {
        QueryWrapper<SysFooterSetup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("footer_name", footerName);
        queryWrapper.eq("site_id", siteId);
        // 添加条件，排除当前ID的记录
        if (id != null) {
            queryWrapper.ne("id", id);
        }
        return service.count(queryWrapper) > 0;
    }


    // 现在仅用于新增场景或编辑时名称变更的校验，不需排除自身ID
    private boolean verifyTheFooterName(String footerName, Integer siteId) {
        QueryWrapper<SysFooterSetup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("footer_name", footerName);
        queryWrapper.eq("site_id", siteId);
        return service.count(queryWrapper) > 0;
    }


    private void handleFloatingColumnSchemeScopeOfApplications(SysFooterSetup footerSetup) {
        if (footerSetup.getFooterSetupScopeOfApplications() != null && !footerSetup.getFooterSetupScopeOfApplications().isEmpty()) {
            for (SysFooterSetupScopeOfApplication scope : footerSetup.getFooterSetupScopeOfApplications()) {
                scope.setFooterId(footerSetup.getId());
                if (footerSetup.getId() == null) {
                    // 如果页脚方案是新建的，直接保存范围
                    footerSetupScopeOfApplicationService.save(scope);
                } else {
                    // 如果页脚方案是更新的，根据情况更新或保存范围
                    footerSetupScopeOfApplicationService.saveOrUpdate(scope);
                }
            }
        }
    }

    /**
     * 保存或更新页脚配置记录
     * 如果ID为空，则执行保存操作；如果ID有值，则视为更新操作。
     *
     * @param sysFooterSetup 要保存或更新的页脚配置对象
     * @return R 结果对象，包含操作状态信息
     */
    @PostMapping("/saveOrUpdate")
    @ApiOperation(value = "保存或更新页脚配置记录", response = SysFooterSetup.class)
    public R saveOrUpdate(@RequestBody @ApiParam(name = "页脚配置对象", value = "包含页脚配置详细信息，若id不存在则新建，否则视为更新", required = true) SysFooterSetup sysFooterSetup) {

        if (sysFooterSetup.getId() != null && sysFooterSetup.getIsDisable() == 0) {
            ConflictCheckResultVO conflictCheckResultVO = service.preCheckMultiScopeConflictBeforeUpdate(sysFooterSetup.getSiteId(),
                    sysFooterSetup.getFooterSetupScopeOfApplications(), sysFooterSetup.getId());
            if (conflictCheckResultVO.isHasConflict()) {
                return R.fail(conflictCheckResultVO.getMessage());
            }
        }

        // 新增场景下直接进行校验
        if (sysFooterSetup.getId() == null) {
            if (verifyTheFooterName(sysFooterSetup.getFooterName(), sysFooterSetup.getSiteId())) {
                return R.fail("该站点下已存在相同的页脚名称");
            }
        }




        // 编辑场景下，只有当名称发生改变时才进行校验，并排除当前记录自身
        else if (!sysFooterSetup.getFooterName().equals(service.getById(sysFooterSetup.getId()).getFooterName())) {
            if (checkSchemeNameExceptSelf(sysFooterSetup.getFooterName(), sysFooterSetup.getSiteId(), sysFooterSetup.getId())) {
                return R.fail("该站点下已存在相同的页脚名称");
            }
        }

        //获取floatingColumnSchemeScopeOfApplications中的范围判断是不是0,如果包含为0
        List<SysFooterSetupScopeOfApplication> footerSetupScopeOfApplications = sysFooterSetup.getFooterSetupScopeOfApplications();
        for (SysFooterSetupScopeOfApplication scopeOfApplication : footerSetupScopeOfApplications) {
            if (scopeOfApplication.getApplicationScopeId().equals("0")) {
                sysFooterSetup.setIsGlobal(1);

            } else {
                sysFooterSetup.setIsGlobal(0);
            }
        }

        // 处理全局页脚逻辑
        if (sysFooterSetup.getIsGlobal() == 1) {
            // 确保没有其他全局页脚同时启用
            long globalCount = service.count(new QueryWrapper<SysFooterSetup>().eq("is_global", true).eq("site_id", sysFooterSetup.getSiteId()).eq("is_disable", 0).ne("id", sysFooterSetup.getId()));
            if (globalCount > 0) {
                return R.fail("已经有全局页脚启用，请先禁用后再尝试设置");
            }
        }

        boolean result = false;


        try {
            if (sysFooterSetup.getId() == null) {
                // 保存操作
                result = service.save(sysFooterSetup);
            } else {
                // 更新操作
                result = service.updateById(sysFooterSetup);
            }
        } catch (Exception e) {
            return R.fail("操作失败");
        }

        if (!result) {
            return R.fail("操作失败");
        }

        //删除原有范围
        footerSetupScopeOfApplicationService.deleteTheRangeBasedOnTheFooterId(sysFooterSetup.getId());

        handleFloatingColumnSchemeScopeOfApplications(sysFooterSetup);

        return R.success();
    }


    @PostMapping("/footerList")
    @ApiOperation(value = "页脚列表", response = FooterListVO.class)
    public R footerList(@RequestBody SuspensionBarListPagingReq sysFloatingColumnScheme) {
        return service.footerList(sysFloatingColumnScheme);
    }


    @PostMapping("/showTheFooterAccordingToTheColumn")
    @ApiOperation(value = "根据栏目展示页脚", response = SysFloatingColumnSchemeVO.class)
    public R columnDisplayFloatingBar(@ApiParam(name = "columnId", value = "栏目id", required = true) @RequestParam String columnId) {
        return service.showTheFooterAccordingToTheColumn(columnId);
    }

    /**
     * * 1.应用全部权限最高，只要是启用应用全部的，除了一开始没有启用页脚的都弹窗提示，当前已启用了的页脚方案将会被取消。
     * 2.启用了应用全部的，后面要启用部分的，提示先禁用应用全部的。
     * 3.部分启用vs部分启用，且有重合的，提示移除重合的。
     */
    @PostMapping("/enableOrDisableScheme")
    @ApiOperation(value = "启用或禁用", response = SysFooterSetup.class)
    public R enableOrDisableScheme(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id, @ApiParam(name = "isDisable", value = "是否禁用(0启用 1禁用)", required = true) @RequestParam Integer isDisable) {
        /* return service.enableOrDisableScheme(id, isDisable);*/

        //禁启用操作
        SysFooterSetup footSetup = service.getById(id);
        footSetup.setIsDisable(isDisable);
        boolean operation = service.updateById(footSetup);
        //增加判断如果当前的应用权限如果为全部，需要将当前站点下其余已经启用的给禁用了
        if (footSetup.getIsDisable() == 0 && footSetup.getIsGlobal() == 1) {
            UpdateWrapper<SysFooterSetup> updateWrapper = new UpdateWrapper<>();
            updateWrapper.set("is_disable", 1);
            updateWrapper.eq("is_disable", 0);
            updateWrapper.eq("site_id", footSetup.getSiteId());
            updateWrapper.ne("id", id);
            service.update(updateWrapper);

        }


        return operation ? R.success() : R.fail("操作失败");
    }


    @PostMapping("/checkEnableOrDisable")
    @ApiOperation(value = "检查“启用”或“禁用”()", response = SysFloatingColumnSchemeVO.class)
    public R checkEnableOrDisable(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id, @ApiParam(name = "isDisable", value = "是否禁用(0启用 1禁用)", required = true) @RequestParam Integer isDisable) {

        return service.checkEnableOrDisable(id, isDisable);
    }


    @PostMapping("/findById")
    @ApiOperation(value = "查询详情", response = SysFooterSetup.class)
    public R findById(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id) {
        SysFooterSetup sysFooterSetup = service.getById(id);

        if (!ObjectUtil.isEmpty(handleFloatingColumnSchemeScopeOfApplications(id))) {
            sysFooterSetup.setFooterSetupScopeOfApplications(handleFloatingColumnSchemeScopeOfApplications(id));
        }

        return R.success(sysFooterSetup);
    }

    //根据页脚获取范围
    private List<SysFooterSetupScopeOfApplication> handleFloatingColumnSchemeScopeOfApplications(Integer footerId) {
        LambdaQueryWrapper<SysFooterSetupScopeOfApplication> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(SysFooterSetupScopeOfApplication::getFooterId, footerId);
        return footerSetupScopeOfApplicationService.list(queryWrapper);
    }


    @PostMapping("/batchDeletion")
    @ApiOperation(value = "批量删除页脚数据", response = SysFloatingColumnScheme.class)
    public R batchDeletion(@ApiParam(name = "ids", value = "主键", required = true) @RequestParam List<Integer> ids) {
        boolean delState = service.removeByIds(ids);
        //删除的同时将页脚方案以及页脚应用范围
        if (delState) {
            for (Integer getFooterId : ids) {
                LambdaQueryWrapper<SysFooterSetupScopeOfApplication> queryWrapper = new LambdaQueryWrapper<>();
                queryWrapper.eq(SysFooterSetupScopeOfApplication::getFooterId, getFooterId);
                footerSetupScopeOfApplicationService.remove(queryWrapper);
            }
        }

        return R.success();
    }
}

