package com.huoran.iasf.controller;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.entity.SysNavigationIcon;
import com.huoran.iasf.service.SysNavigationIconService;
import com.huoran.iasf.vo.req.SortOrderReq;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * @描述：导航图标设置控制类
 * @作者: Rong
 * @日期: 2024-07-04
 */
@RestController
@RequestMapping("/SysNavigationIconService/SysNavigationIcon")
@Api(value = "导航图标设置:SysNavigationIconController", tags = "A-导航图标设置:导航图标设置")
public class SysNavigationIconController {

    @Autowired
    public SysNavigationIconService service;


    @PostMapping("/checkIfTheTitleIsRepeat")
    @ApiOperation(value = "标题判重——适用于新增、编辑及发布前的校验", response = Boolean.class)
    public R checkIfTheTitleIsRepeat(@RequestBody @Valid SysNavigationIcon icon) {
        boolean isDuplicate = false;

        // 如果ID不为空，说明是编辑操作，需要排除当前记录自身
        if (icon.getId() != null) {
            isDuplicate = service.existsOtherByTitle(icon.getTitle(), icon.getId(), icon.getSiteId());
        } else {
            // ID为空，说明是新增操作
            isDuplicate = service.existsByTitle(icon.getTitle(), icon.getSiteId());
        }

        return R.result(!isDuplicate, !isDuplicate ? "标题可用" : "该标题已重复!");
    }

    /**
     * 保存或更新导航图标设置记录
     * 如果ID为空，则执行保存操作；如果ID有值，则视为更新操作。
     *
     * @param sysNavigationIcon 要保存或更新的导航图标设置对象
     * @return R 结果对象，包含操作状态信息
     */
    @PostMapping("/saveOrUpdate")
    @ApiOperation(value = "保存或更新导航图标设置记录", response = SysNavigationIcon.class)
    public R saveOrUpdate(@RequestBody @ApiParam(name = "导航图标设置对象", value = "包含导航图标设置详细信息，若id不存在则新建，否则视为更新", required = true) SysNavigationIcon sysNavigationIcon) {

        if (sysNavigationIcon.getId() == null) {
            // 保存操作
            boolean addState = service.save(sysNavigationIcon);
            return addState ? R.success(sysNavigationIcon.getId()) : R.fail("新增失败");
        } else {
            // 更新操作
            boolean updateState = service.updateById(sysNavigationIcon);
            return updateState ? R.success() : R.fail("编辑失败");
        }
    }

    @PostMapping("/updateSortOrder")
    @ApiOperation(value = "批量更新导航图标的排序顺序", notes = "接收一个包含ID和对应排序值的列表")
    public R updateSortOrder(@RequestBody List<SortOrderReq> sortOrderRequests) {
        // 调用服务层方法来批量更新排序值
        service.batchUpdateSortOrder(sortOrderRequests);

        // 返回操作成功的结果
        return R.success("排序顺序更新成功");
    }


    @PostMapping("/list")
    @ApiOperation(value = "根据站点ID查询列表数据，可选过滤禁用状态", response = SysNavigationIcon.class, responseContainer = "List")
    public R listSysNavigationIconsByCondition(@ApiParam(value = "站点ID", required = true) @RequestParam Integer siteId, @ApiParam(value = "禁用状态：0-启用，1-禁用，默认查询所有状态") @RequestParam(required = false) Integer isDisable,
                                               @ApiParam(value = "查询关键词") @RequestParam(required = false) String search) {
        List<SysNavigationIcon> sysNavigationIcons = service.listBySiteIdAndStatus(siteId, isDisable,search);
        return R.success(sysNavigationIcons);
    }


    @PostMapping("/findById")
    @ApiOperation(value = "查询详情", response = SysNavigationIcon.class)
    public R findById(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id) {
        SysNavigationIcon sysNavigationIcon = service.getById(id);
        return R.success(sysNavigationIcon);
    }


    @PostMapping("/batchDeletion")
    @ApiOperation(value = "批量删除", response = SysNavigationIcon.class)
    public R batchDeletion(@ApiParam(name = "ids", value = "主键", required = true) @RequestParam List<Integer> ids) {
        boolean delState = service.removeByIds(ids);
        return delState ? R.success() : R.fail("删除失败");
    }


    @PostMapping("/enableDisable")
    @ApiOperation(value = "启用禁用", response = SysNavigationIcon.class)
    public R enableDisable(@ApiParam(name = "id", value = "主键id", required = true) @RequestParam Integer id, @ApiParam(name = "isDisable", value = "是否禁用：0-启用，1-禁用", required = true) @RequestParam Integer isDisable) {
        UpdateWrapper<SysNavigationIcon> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("is_disable", isDisable);
        updateWrapper.eq("id", id);
        boolean ret = service.update(updateWrapper);
        return ret ? R.success() : R.fail("禁用/启用失败");
    }
}

