package com.huoran.iasf.entity;

import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * @描述：综合产品与课程详情信息表
 * @作者: Rong
 * @日期: 2024-06-13
 */
@Data
@ApiModel(value = "综合产品与课程详情信息表")
@TableName("sys_product_details")
public class ProductDetails implements Serializable {

    private static final long serialVersionUID = 1L;

    @TableId(type = IdType.AUTO)
    @ApiModelProperty(value = "主键ID")
    private Integer id;

    @ApiModelProperty(value = "栏目id")
    private Integer columnId;

    @ApiModelProperty(value = "名称，课程或产品的名称，根据类型区分")
    private String name;

   /* @ApiModelProperty(value = "类型标识，区分是课程还是产品")
    private String type;*/

    @ApiModelProperty(value = "课程分类id或产品分类id，根据类型使用")
    private String categoryId;

    @ApiModelProperty(value = "产品分类标签id，根据类型区分使用")
    private String classificationTagId;

    @ApiModelProperty(value = "封面图片URL，课程封面或产品封面")
    private String coverImageUrl;

    @ApiModelProperty(value = "来源信息，仅课程详情使用，可为空")
    private String source;

    @ApiModelProperty(value = "按钮跳转名称")
    private String jumpButtonName;

    @ApiModelProperty(value = "按钮跳转链接")
    private String jumpLinkUrl;

    @ApiModelProperty(value = "简介")
    private String introduction;

    @ApiModelProperty(value = "详细描述")
    private String detailedDescription;

    @ApiModelProperty(value = "发布状态，0草稿，1已发布")
    private Boolean publishStatus;

    @ApiModelProperty(value = "创建时间")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    @ApiModelProperty(value = "更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    @ApiModelProperty(value = "是否删除(1未删除；0已删除)")
    @TableLogic
    private Integer deleted;

    @ApiModelProperty(value = "课程类型(免费、付费)，仅课程使用，可为空")
    private String courseType;

    @ApiModelProperty(value = "匹配课程，仅产品使用，可存储课程ID列表")
    private String matchedCourses;

    @ApiModelProperty(value = "预计课时，仅产品使用，可为空")
    private String estimatedHours;

    @ApiModelProperty(value = "适用场景，仅产品使用，可为空")
    private String applicableScenarios;

    @ApiModelProperty(value = "创建人id")
    private Integer founderId;

    @ApiModelProperty(value = "编辑人id")
    private Integer editorId;

    @ApiModelProperty(value = "文章总浏览量")
    private Integer totalBrowsing;

    @ApiModelProperty(value = "站点id")
    @NotNull(message = "站点id不能为空")
    private Integer siteId;

    @ApiModelProperty(value = "文章模板(前端自定义)")
    private Integer articleTemplate;

    @ApiModelProperty(value = "模板状态(0禁用；1启用)")
    private Integer templateStatus;

    @ApiModelProperty(value = "是否置顶(默认为0 不置顶 1为置顶)")
    private Integer isTop;

    @ApiModelProperty(value = "置顶时间")
    private Date topTime;

    @ApiModelProperty(value = "发布时间")
    private String releaseTime;



    @ApiModelProperty(value = "链接类型")
    private Integer connectionType;

    @ApiModelProperty(value = "链接地址")
    private String linkAddress;

    @ApiModelProperty(value = "站点选择(连接类型为其它站点连接时储存)")
    private Integer siteSelection;

    @ApiModelProperty(value = "是否在新窗口打开(0:否 1:是)")
    private Integer isOpen;

    @ApiModelProperty(value = "是否禁用(0默认，0启用 1禁用)")
    private Integer isDisable;











    @ApiModelProperty(value = "学科专业类")
    @TableField(exist = false)
    private List<SysSubjectSpecialty> subjectSpecialtyList;

    @ApiModelProperty(value = "学科类别")
    @TableField(exist = false)
    private String disciplineName;

    @ApiModelProperty(value = "专业类")
    @TableField(exist = false)
    private String professionalClassName;

    @ApiModelProperty(value = "专业")
    @TableField(exist = false)
    private String professionalName;

    @ApiModelProperty(value = "栏目名称")
    @TableField(exist = false)
    private String columnName;


}