package com.huoran.iasf.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.common.utils.TreeStructureUtils;
import com.huoran.iasf.entity.SysColumn;
import com.huoran.iasf.mapper.SysColumnMapper;
import com.huoran.iasf.mapper.SysContentMapper;
import com.huoran.iasf.service.SysColumnService;
import com.huoran.iasf.vo.FatherContentRespVO;
import com.huoran.iasf.vo.req.ColumnWeightReqVO;
import com.huoran.iasf.vo.req.LongPageColumnReqVO;
import com.huoran.iasf.vo.req.PaginationColumnReqVO;
import com.huoran.iasf.vo.resp.PageContentRespVO;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @描述： 服务类
 * @作者: Rong
 * @日期: 2022-08-01
 */
@Service
public class SysColumnServiceImpl extends ServiceImpl<SysColumnMapper, SysColumn> implements SysColumnService {

    @Autowired
    private SysColumnMapper mapper;

    @Autowired
    private SysContentMapper contentMapper;

    @Override
    public boolean sameLevelJudgment(ColumnWeightReqVO column) {
        QueryWrapper<SysColumn> queryWrapper = new QueryWrapper<>();


        queryWrapper.eq("column_name", column.getColumnName());

        if (column.getId() != null) {
            queryWrapper.eq("father_id", column.getId());
        } else {
            queryWrapper.eq("father_id", 0);
        }


        queryWrapper.eq("site_id", column.getSiteId());
        queryWrapper.eq("deleted", 1);
        SysColumn sysColumn = mapper.selectOne(queryWrapper);
        if (sysColumn != null) {
            if (column.getId() != null) {
                //判断为编辑情况下，如果编辑的id与查询出来的一致不提示
                if (column.getId() == sysColumn.getId()) {
                    return true;
                }
            }

            return false;
        }
        return true;
    }


    @Override
    public List<SysColumn> listWithTree(PaginationColumnReqVO column) {
        //查询所有栏目
        List<SysColumn> columnList = baseMapper.filter(column);
        /*// 创建树形结构（数据集合作为参数）
        TreeBuild treeBuild = new TreeBuild(columnList);
        // 原查询结果转换树形结构
        columnList = treeBuild.buildTree();
        // AjaxResult：个人封装返回的结果体
        return  columnList;*/
        return forMethod(columnList);

    }

    /**
     * 双重for循环方法转换成树形结构
     *
     * @param treeList
     * @return
     */
    public static List<SysColumn> forMethod(List<SysColumn> treeList) {
        List<SysColumn> rootTree = new ArrayList<>();
        for (SysColumn tree : treeList) {
            // 第一步 筛选出最顶级的父节点
            if (0 == tree.getFatherId()) {
                rootTree.add(tree);
            }
            // SysColumn 筛选出该父节点下的所有子节点列表
            for (SysColumn node : treeList) {
                if (node.getFatherId().equals(tree.getId())) {
                    if (CollectionUtils.isEmpty(tree.getChildren())) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(node);
                }
            }
        }
        return rootTree;
    }

    @Override
    public List<SysColumn> listWithTreeMenuVisible(PaginationColumnReqVO column) {

        if (column.getRole() == null) {

        } else if (column.getRole() != null || column.getRole().equals("admin") || ObjectUtil.isNotNull(column.isIsadmin()) || column.isIsadmin()) {
            throw new IllegalArgumentException("参数名不能包含admin");
        }
        //查询所有栏目
        List<SysColumn> columnList = baseMapper.filterMenuVisible(column);

        if (column.getColumnName().equals("")) {
            //用来保存父节点
            List<SysColumn> rootColumn = new ArrayList<>();
            //用来保存子节点
            List<SysColumn> childList = new ArrayList<>();
            //遍历所有栏目，将所有栏目的父栏目id为0的栏目放入父节点集合中
            for (SysColumn column1 : columnList) {
                Integer parentId = column1.getFatherId();
                if (parentId == 0) {
                    rootColumn.add(column1);
                }
            }
            for (SysColumn sysColumn : rootColumn) {
                Integer theColumnId = sysColumn.getId();
                //调用工具类获取子节点
                childList = TreeStructureUtils.getChildren(theColumnId, columnList);
                sysColumn.setChildren(childList);
            }

            return rootColumn;
        }

        return columnList;

    }

    @Override
    public R longPageColumnList(LongPageColumnReqVO reqVO) {
        Page<PageContentRespVO> page = new Page<PageContentRespVO>(reqVO.getPageNum(), reqVO.getPageSize());
        IPage<PageContentRespVO> pageList = baseMapper.longPageColumnList(page, reqVO);
        pageList.getRecords().forEach(pageContentRespVO -> {
            if (pageContentRespVO.getLevel() > 1) {
                //判断该栏目层级为第一级时候不返回上级栏目
                List<FatherContentRespVO> upperLevel =
                        baseMapper.getParentInformationBasedOnChild(pageContentRespVO.getId());
                pageContentRespVO.setSuperiorColumn(upperLevel);
            }

        });


        return R.success(pageList);
    }

    @Override
    public R getsTheSubColumn(Integer pid, String ids, Integer siteId) {

        List<SysColumn> getsTheSubColumn = baseMapper.getsTheSubColumn(pid, ids, null, siteId);
        return R.success(TreeStructureUtils.buildTree(getsTheSubColumn, pid));
    }

    @Override
    public R getsSublevelColumnsUnderALevel(Integer parentId, String ids, Integer siteId) {

        List<SysColumn> getsTheSubColumn = baseMapper.getsTheSubColumn(parentId, ids, null, siteId);
        List<SysColumn> buildTree = TreeStructureUtils.buildTree(getsTheSubColumn, parentId);
        return R.success(buildTree);
    }

    @Override
    public Integer oneLevelChecksThemAll(Integer theChildId) {
        Integer getPid = baseMapper.oneLevelChecksThemAll(theChildId);
        return getPid;
    }

    @Override
    public R getSubColumn(Integer parentId, String ids, Integer isSort, Integer siteId) {
        List<SysColumn> getsTheSubColumn = baseMapper.getsTheSubColumn(parentId, ids, isSort, siteId);
        return R.success(TreeStructureUtils.forMethod(getsTheSubColumn));
    }

    /**
     * 根据栏目类型ID查询文章。
     * <p>
     * 此方法用于根据提供的栏目类型ID字符串查询相关文章。如果ID字符串包含多个ID，则会考虑这些ID的数量来决定查询策略。
     * 如果ID数量大于1，且查询到的栏目类型数量也为1，则会应用额外的文章过滤条件。
     * </p>
     *
     * @param ids 栏目类型ID字符串，多个ID以逗号分隔。
     * @param id  单个栏目类型ID，用于当提供的ID字符串包含单个ID时的查询。
     * @return 返回查询结果，封装在R对象中。
     */
    /*@Override
    public R queryArticlesByColumnType(String ids, Integer id) {
        // 计算提供的ID字符串中包含的栏目类型ID的数量。
        Integer count = 0;
        for (String columnId : ids.split(",")) {
            ++count;
        }
        SysColumn column = baseMapper.selectById(id);
        if (!ObjectUtil.isEmpty(column) && (column.getTemplateId() == 12 || (column.getTemplateId() == 9 && column
        .getListStyleId() == 73))) {
            // 根据模板ID为12的特定情况调用相应方法
            return null;
        } else {
            // 根据提供的栏目类型ID字符串查询栏目类型。
            List<SysColumn> queryByColumnType = baseMapper.queryByColumnType(ids);
            // 如果提供的ID字符串中包含的栏目类型ID数量大于1，并且查询到的栏目类型数量为1，则应用额外的过滤条件。
            //为1表示该层级下以及它本身列表样式模板一直则查询全部的
            if (count > 1) {
                if (queryByColumnType.size() == 1) {
                    // 使用过滤条件查询文章。
                    return R.success(contentMapper.columnConditionsFilterArticles(ids));
                }
            }
            // 在其他情况下，直接根据单个栏目类型ID查询文章。
            //表示该层级下的全部层级不一致只查本身
            return R.success(contentMapper.columnConditionsFilterArticles(id + ""));
        }


    }*/


    // 常量定义
    private static final int TEMPLATE_ID_12 = 12;
    private static final int TEMPLATE_ID_9 = 9;
    private static final int LIST_STYLE_ID_73 = 73;

    @Override
    public R queryArticlesByColumnType(String ids, Integer id) {
        // 检查输入参数是否有效
        if (StrUtil.isBlank(ids)) {
            return R.fail("栏目类型ID字符串不能为空");
        }
        ids = ids.trim();

        // 初始化计数器
        long count = Arrays.stream(ids.split(",")).filter(str -> !str.isEmpty()).count();

        SysColumn column;
        List<SysColumn> queryByColumnType;
        try {
            column = baseMapper.selectById(id);
            queryByColumnType = baseMapper.queryByColumnType(ids);
        } catch (Exception e) {
            return R.fail("查询栏目类型时发生异常");
        }

        if (column != null && (column.getTemplateId() == TEMPLATE_ID_12 || (column.getTemplateId() == TEMPLATE_ID_9 && column.getListStyleId() == LIST_STYLE_ID_73))) {
            if (count > 1) {

                List<PageContentRespVO> list = contentMapper.columnConditionsFilterProductCourse(ids);
                return R.success(list);
            } else {
                List<PageContentRespVO> list = contentMapper.columnConditionsFilterProductCourse(String.valueOf(id));
                return R.success(list);
            }
        }

        if (count > 1 && queryByColumnType.size() == 1) {
            return R.success(contentMapper.columnConditionsFilterArticles(ids));
        }

        return R.success(contentMapper.columnConditionsFilterArticles(id.toString()));
    }


    @Override
    public List<String> getColumnNamesByIds(Collection<String> applicationScopeIds) {
        if (CollectionUtils.isEmpty(applicationScopeIds)) {
            return Collections.emptyList();
        }

        // 将ID集合转换为逗号分隔的字符串，以便在IN子句中使用
        String scopeIdsStr = String.join(",", applicationScopeIds);

        // 执行SQL查询，这里直接使用BaseMapper的selectList方法并传入自定义的Wrapper
        QueryWrapper<SysColumn> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", applicationScopeIds);
        queryWrapper.select("column_name"); // 只选择columnName列

        return baseMapper.selectObjs(queryWrapper).stream().map(Object::toString) // 假设columnName为String类型，转换对象为String
                .collect(Collectors.toList());
    }


}



