package com.huoran.iasf.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.entity.SysColumn;
import com.huoran.iasf.entity.SysFloatingColumnScheme;
import com.huoran.iasf.entity.SysFloatingColumnSchemeModule;
import com.huoran.iasf.entity.SysFloatingColumnSchemeScopeOfApplication;
import com.huoran.iasf.mapper.SysColumnMapper;
import com.huoran.iasf.mapper.SysFloatingColumnSchemeMapper;
import com.huoran.iasf.mapper.SysFloatingColumnSchemeModuleMapper;
import com.huoran.iasf.mapper.SysFloatingColumnSchemeScopeOfApplicationMapper;
import com.huoran.iasf.service.SysFloatingColumnSchemeService;
import com.huoran.iasf.vo.ConflictCheckResultVO;
import com.huoran.iasf.vo.SysFloatingColumnSchemeScopeOfApplicationWithSiteIdVO;
import com.huoran.iasf.vo.SysFloatingColumnSchemeVO;
import com.huoran.iasf.vo.req.SuspensionBarListPagingReq;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 悬浮栏方案 服务实现类
 * </p>
 *
 * @author cheney
 * @since 2024-06-04
 */
@Service
public class SysFloatingColumnSchemeServiceImpl extends ServiceImpl<SysFloatingColumnSchemeMapper, SysFloatingColumnScheme> implements SysFloatingColumnSchemeService {

    @Resource
    private SysFloatingColumnSchemeMapper floatingColumnSchemeMapper;

    @Autowired
    public SysFloatingColumnSchemeScopeOfApplicationMapper scopeOfApplicationMapper;

    @Autowired
    public SysFloatingColumnSchemeModuleMapper floatingColumnSchemeModuleMapper;

    @Autowired
    public SysColumnMapper columnMapper;

    private Log logger = LogFactory.getLog(getClass());

    @Override
    public R floatingBarList(SuspensionBarListPagingReq reqVO) {
        Page<SysFloatingColumnSchemeVO> page = new Page<SysFloatingColumnSchemeVO>(reqVO.getPageNum(), reqVO.getPageSize());
        IPage<SysFloatingColumnSchemeVO> pageList = baseMapper.floatingBarList(page, reqVO);
        return R.success(pageList);
    }


    /**
     * isGlobal为1表示应用全部的意思，为1表示部分应用。(表示是否为全局应用，便于快速判断是否存在全局悬浮栏,0表示不是全局悬浮栏，1表示是全局悬浮栏)
     * <p>
     * <p>
     * <p>
     * 应用全部权限最高。
     * 1.只要是启用应用全部的，除了没有启用的悬浮栏,包含全部，不论是应用全部还是部分应用的情况下,都会弹窗提示:"当前已启用了的悬浮栏方案将会被取消!"
     * 2.如果当前操作的悬浮栏是要为启用状态，且当前的悬浮栏应用类型为为部分应用的情况下下，需要判断当前站点下是否存在已经启用了应用全部的，如果存在，则弹窗提示:"当前已启用全局悬浮栏，无法启用部分应用的悬浮栏，请先禁用全局悬浮栏！"
     * 3.部分启用vs部分启用，且有重合的，提示"当前选择的栏目范围与已启用的悬浮栏方案有冲突，请移除冲突的栏目后再试：{后边展示的是冲突的栏目名称}"
     *
     * @param schemeId 悬浮栏目id
     * @param enable   是否禁用(0默认，0启用 1禁用)
     * @return
     */
    @Transactional
    @Override
    public R enableOrDisableScheme(Integer schemeId, Integer enable) {
       /* SysFloatingColumnScheme scheme = floatingColumnSchemeMapper.selectById(schemeId);
        scheme.setFloatingColumnSchemeScopeOfApplications(scopeOfApplicationMapper.selectList(new QueryWrapper<SysFloatingColumnSchemeScopeOfApplication>().lambda().eq(SysFloatingColumnSchemeScopeOfApplication::getFloatingBarSchemeId, schemeId)));

        if (ObjectUtil.isEmpty(scheme)) {
            return R.fail("悬浮窗方案不存在");
        }

        // 获取当前悬浮栏是否为全局应用（1表示全局，0表示非全局）
        Integer isGlobal = scheme.getIsGlobal();

        // 首先处理启用全局悬浮栏逻辑
        if (enable == 0 && isGlobal.equals(1)) {
            // 确保没有其他全局悬浮栏已启用
            SysFloatingColumnScheme existingGlobal = floatingColumnSchemeMapper.selectOne(new LambdaQueryWrapper<SysFloatingColumnScheme>().eq(SysFloatingColumnScheme::getIsGlobal, 1).eq(SysFloatingColumnScheme::getIsDisable, 0));
            if (existingGlobal != null && !Objects.equals(existingGlobal.getId(), schemeId)) {
                System.err.println("已有其他全局悬浮栏方案启用，请先禁用它！");
            }
        }

        // 2. 当尝试启用非全局悬浮栏，检查是否与已启用的全局悬浮栏冲突
        if (enable == 0) {
            SysFloatingColumnScheme globalEnabled = floatingColumnSchemeMapper.selectOne(new LambdaQueryWrapper<SysFloatingColumnScheme>().eq(SysFloatingColumnScheme::getIsGlobal, 1).eq(SysFloatingColumnScheme::getIsDisable, 0));
            if (globalEnabled != null) {
                System.err.println("当前已启用全局悬浮栏，无法启用部分应用的悬浮栏，请先禁用全局悬浮栏");
            }
        }

        // 3. 部分启用vs部分启用的冲突检查
        if (enable == 0) {
            List<SysFloatingColumnScheme> activeSchemes = floatingColumnSchemeMapper.selectList(new LambdaQueryWrapper<SysFloatingColumnScheme>().ne(SysFloatingColumnScheme::getId, schemeId).eq(SysFloatingColumnScheme::getIsDisable, 0));

            if (!activeSchemes.isEmpty()) {
                Set<String> currentScopeIdsSet = new HashSet<>();
                for (SysFloatingColumnSchemeScopeOfApplication scope : scheme.getFloatingColumnSchemeScopeOfApplications()) {
                    currentScopeIdsSet.addAll(Arrays.asList(scope.getApplicationScopeId().split(",")));
                }

                List<SysFloatingColumnSchemeScopeOfApplication> activeScopes = scopeOfApplicationMapper.selectList(new LambdaQueryWrapper<SysFloatingColumnSchemeScopeOfApplication>().in(SysFloatingColumnSchemeScopeOfApplication::getFloatingBarSchemeId, activeSchemes.stream().map(SysFloatingColumnScheme::getId).collect(Collectors.toList())));

                List<String> conflictingScopeIds = activeScopes.stream().flatMap(scope -> Arrays.stream(scope.getApplicationScopeId().split(","))).distinct().filter(currentScopeIdsSet::contains).collect(Collectors.toList());

                if (!conflictingScopeIds.isEmpty()) {
                    //根据获取到的栏目id 通过in查询栏目的名称
                    List<String> conflictingScopeNames = columnMapper.selectList(new LambdaQueryWrapper<SysColumn>().in(SysColumn::getId, conflictingScopeIds)).stream().map(SysColumn::getColumnName).collect(Collectors.toList());
                    String columnName = String.join(",", conflictingScopeNames);
                    System.err.println("当前选择的栏目范围与已启用的悬浮栏方案有冲突，请移除冲突的栏目后再试：" + columnName);
                }
            }
        }

        // 更新悬浮栏方案的状态
        scheme.setIsDisable(enable);
        floatingColumnSchemeMapper.updateById(scheme);

        // 如果启用的是全局悬浮栏，需要禁用其他已启用的非全局悬浮栏
        if (isGlobal.equals(1) && enable == 0) {
            getNonGlobal(schemeId);
        }*/

        return R.success("操作成功");
    }


    /**
     * 这个方法用来检查是否可以启用或禁用指定悬浮栏方案
     * isGlobal为1表示应用全部的意思，为1表示部分应用。(表示是否为全局应用，便于快速判断是否存在全局悬浮栏,0表示不是全局悬浮栏，1表示是全局悬浮栏)
     * <p>
     * --所有操作里面应用全部权限最高。
     * 1.只要是启用应用全部的，除了没有启用的悬浮栏,包含全部，不论是应用全部还是部分应用的情况下,都会弹窗提示:"当前已启用了的悬浮栏方案将会被取消!"
     * 2.如果当前操作的悬浮栏是要为启用状态，且当前的悬浮栏应用类型为为部分应用的情况下下，需要判断当前站点下是否存在已经启用了应用全部的，如果存在，则弹窗提示:"当前已启用全局悬浮栏，无法启用部分应用的悬浮栏，请先禁用全局悬浮栏！"
     * 3.部分启用vs部分启用，且有重合的，提示"当前选择的栏目范围与已启用的悬浮栏方案有冲突，请移除冲突的栏目后再试：{后边展示的是冲突的栏目名称}"
     *
     * @param schemeId 悬浮栏目id
     * @param enable   是否禁用(0启用 1禁用)
     * @return 包含操作结果或警告信息的对象
     */

    public R checkEnableOrDisable(Integer schemeId, Integer enable) {
        SysFloatingColumnScheme scheme = floatingColumnSchemeMapper.selectById(schemeId);
        if (scheme == null) {
            return R.fail("悬浮窗方案不存在");
        }

        Integer siteId = scheme.getSiteId();

        // 1. 当前操作为启用时，检查逻辑
        if (enable == 0) {
            // 全局悬浮栏启用逻辑
            // 对于启用操作，先检查是否是全局应用
            if (scheme.getIsGlobal() == 1) {
                // 检查站点下是否存在任何已启用的悬浮栏方案，不论全局还是部分应用
                long enabledCount = floatingColumnSchemeMapper.selectCount(new LambdaQueryWrapper<SysFloatingColumnScheme>().eq(SysFloatingColumnScheme::getSiteId, siteId).eq(SysFloatingColumnScheme::getIsDisable, 0));
                if (enabledCount > 0) {
                    // 弹窗提示，表明启用新的全局悬浮栏会覆盖当前已启用的悬浮栏
                    return R.warn("当前站点已存在启用的悬浮栏方案，启用新方案将会覆盖原有设置，请确认是否继续？");
                }
            } else { // 非全局悬浮栏启用逻辑
                // 检查是否存在启用的全局悬浮栏
                if (existsEnabledGlobal(siteId)) {
                    return R.fail("当前已启用全局悬浮栏，无法直接启用部分应用的悬浮栏，请先禁用全局悬浮栏！");
                }
                // 检查与其它部分应用悬浮栏的冲突
                if (hasOverlapWithOtherEnabled(schemeId, siteId)) {
                    return R.fail(generateConflictMessage(schemeId, siteId, enable));
                }
            }
        } else if (enable == 1) { // 禁用操作逻辑
            // 禁用操作直接允许，无需特殊检查
        } else {
            return R.fail("无效的操作类型");
        }

        // 如果没有冲突，直接返回成功提示
        return R.success("操作可执行");
    }

    @Override
    public R columnDisplayFloatingBar(String columnId) {
        List<SysFloatingColumnSchemeModule> columnSchemeModuleList = new ArrayList<>();
        //获取站点
        SysColumn column = columnMapper.selectById(columnId);
        if (ObjectUtil.isEmpty(column)) {
            return R.fail("未找到该栏目");
        }


        //查询当前站点下已启用，且已发布的悬浮栏
        List<SysFloatingColumnScheme> floatingColumnSchemeList = baseMapper.selectList(new LambdaQueryWrapper<SysFloatingColumnScheme>().eq(SysFloatingColumnScheme::getSiteId, column.getSiteId()).eq(SysFloatingColumnScheme::getIsDisable, 0).eq(SysFloatingColumnScheme::getStatus, 1));

        for (SysFloatingColumnScheme scheme : floatingColumnSchemeList) {
            //判断当前启用的栏目如果为全局的直接返回全局的已启用的方案
            if (scheme.getIsGlobal() == 1) {
                //查询悬浮栏当前绑定的方案
                List<SysFloatingColumnSchemeModule> columnSchemeModule = floatingColumnSchemeModuleMapper.selectList(new LambdaQueryWrapper<SysFloatingColumnSchemeModule>().eq(SysFloatingColumnSchemeModule::getFloatingBarSchemeId, scheme.getId()).eq(SysFloatingColumnSchemeModule::getIsDisable, 0));
                ;
                if (!ObjectUtil.isEmpty(columnSchemeModule)) {
                    return R.success(columnSchemeModule);
                }
            } else {
                //查询栏目绑定过的悬浮栏
                List<Integer> floatingBarIds = scopeOfApplicationMapper.checkBoundByColumnId(columnId);
                //查询悬浮栏绑定的方案
                for (Integer floatingBarId : floatingBarIds) {
                    List<SysFloatingColumnSchemeModule> list = floatingColumnSchemeModuleMapper.selectList(new LambdaQueryWrapper<SysFloatingColumnSchemeModule>().eq(SysFloatingColumnSchemeModule::getFloatingBarSchemeId, floatingBarId).eq(SysFloatingColumnSchemeModule::getIsDisable, 0));
                    columnSchemeModuleList.addAll(list);
                }


                return R.success(columnSchemeModuleList);


            }

        }


        return R.success(columnSchemeModuleList);
    }


    /**
     * 在多个悬浮栏方案更新前，预先检查是否存在应用范围冲突。
     *
     * @param siteId        当前操作所在的站点ID。
     * @param updatedScopes 待更新的悬浮栏方案列表。
     * @return 冲突检测结果，包含是否有冲突及详细的冲突信息。
     */
    @Transactional(readOnly = true)
    public ConflictCheckResultVO preCheckMultiScopeConflictBeforeUpdate(
            Integer siteId, List<SysFloatingColumnSchemeScopeOfApplication> updatedScopes,Integer floatingBarSchemeId) {

        // 1. 收集待更新方案的ID，用于排除这些方案在冲突检测中的干扰
        List<String> excludeSchemeIds = updatedScopes.stream()
                .map(SysFloatingColumnSchemeScopeOfApplication::getApplicationScopeId)
                .collect(Collectors.toList());

        // 2. 从数据库查询该站点下所有启用的悬浮栏方案，但排除掉当前正在更新的那些方案
        List<SysFloatingColumnSchemeScopeOfApplicationWithSiteIdVO> enabledScopes =
                baseMapper.selectEnabledScopesForMultiConflictCheck(siteId, excludeSchemeIds,floatingBarSchemeId);

        /*// 初始化冲突标记为false，并准备一个StringBuilder用于记录冲突信息
        boolean hasConflict = false;
        StringBuilder conflictMessage = new StringBuilder();

        // 3. 对比每个待更新的方案与已启用方案，检测应用范围是否有重叠
        for (SysFloatingColumnSchemeScopeOfApplicationWithSiteIdVO existingScope : enabledScopes) {
            for (SysFloatingColumnSchemeScopeOfApplication newScope : updatedScopes) {
                // 假设冲突基于应用范围的直接比较，实际情况可能需要更复杂的逻辑
                if (existingScope.getApplicationScopeId().equals(newScope.getApplicationScopeId())) {
                    hasConflict = true;
                    conflictMessage.append("发现冲突：方案“")
                            .append(existingScope.getSchemeName())
                            .append("”与待更新的方案在应用范围“")
                            .append(newScope.getApplicationScopeId())
                            .append("”上重叠。\n");
                }
            }
        }*/
        // 初始化冲突标记为false，并准备一个StringBuilder用于记录冲突信息
        boolean hasConflict = false;
        StringBuilder conflictMessageBuilder = new StringBuilder();
        Set<String> conflictColumnNames = new HashSet<>(); // 用于存储所有冲突的栏目名称

        // 3. 对比每个待更新的方案与已启用方案，检测应用范围是否有重叠
        for (SysFloatingColumnSchemeScopeOfApplicationWithSiteIdVO existingScope : enabledScopes) {
            for (SysFloatingColumnSchemeScopeOfApplication newScope : updatedScopes) {
                // 分割应用范围ID为集合，以便逐个比较
                Set<Integer> existingScopeIds = Arrays.stream(existingScope.getApplicationScopeId().split(","))
                        .map(String::trim)
                        .filter(id -> !id.isEmpty() && NumberUtils.isCreatable(id))
                        .map(Integer::parseInt)
                        .collect(Collectors.toSet());
                Set<Integer> newScopeIds = Arrays.stream(newScope.getApplicationScopeId().split(","))
                        .map(String::trim)
                        .filter(id -> !id.isEmpty() && NumberUtils.isCreatable(id))
                        .map(Integer::parseInt)
                        .collect(Collectors.toSet());

                // 找出重叠的范围ID
                Set<Integer> overlapIds = new HashSet<>(existingScopeIds);
                overlapIds.retainAll(newScopeIds);

                // 避免重复处理相同的冲突范围
                if (!overlapIds.isEmpty()) {
                    for (Integer overlapId : overlapIds) {
                        String columnName = getColumnNameById(String.valueOf(overlapId)); // 假设这是获取栏目名称的方法
                        conflictColumnNames.add(columnName); // 收集冲突的栏目名称
                    }
                }
            }
        }

        // 构建最终的冲突信息，如果存在冲突
        if (!conflictColumnNames.isEmpty()) {
            hasConflict = true;
            conflictMessageBuilder.append("栏目：");
            boolean isFirst = true;
            for (String columnName : conflictColumnNames) {
                if (!isFirst) {
                    conflictMessageBuilder.append("、"); // 使用中文逗号作为分隔符
                }
                conflictMessageBuilder.append(columnName);
                isFirst = false;
            }
            conflictMessageBuilder.append(" 已设置有悬浮栏，请先移除相关设置再尝试启用。\n");
        }
        // 4. 根据检测结果创建并返回ConflictCheckResult对象
        return new ConflictCheckResultVO(hasConflict, conflictMessageBuilder.toString());
    }


    // 辅助方法检查站点下是否存在启用的全局悬浮栏
    private boolean existsEnabledGlobal(Integer siteId) {

        QueryWrapper<SysFloatingColumnScheme> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_global", 1);
        queryWrapper.eq("is_disable", 0);
        queryWrapper.eq("site_id", siteId);
        return !floatingColumnSchemeMapper.selectList(queryWrapper).isEmpty();

    }


    /**
     * 检查当前悬浮栏方案与已启用方案是否有重叠。
     *
     * @param schemeId 悬浮栏方案ID
     * @return 是否有重叠
     */
    private boolean hasOverlapWithOtherEnabled(Integer schemeId, Integer siteId) {
        // 获取当前要检查的方案的应用范围
        List<SysFloatingColumnSchemeScopeOfApplication> currentScopes = scopeOfApplicationMapper.selectList(new LambdaQueryWrapper<SysFloatingColumnSchemeScopeOfApplication>().eq(SysFloatingColumnSchemeScopeOfApplication::getFloatingBarSchemeId, schemeId));

        if (currentScopes.isEmpty()) {
            return false; // 当前方案无应用范围，自然无重叠
        }

        Set<Integer> currentScopeIds = currentScopes.stream().flatMap(scope -> Arrays.stream(scope.getApplicationScopeId().split(","))).map(Integer::parseInt).collect(Collectors.toSet());

        // 构造查询条件，确保比较的方案属于同一个站点
        String sqlInCondition = "SELECT id FROM sys_floating_column_scheme WHERE is_disable = 0 AND site_id = " + siteId;

        // 查询所有启用且ID不等于当前schemeId且属于相同站点的悬浮栏方案的应用范围
        List<SysFloatingColumnSchemeScopeOfApplication> enabledScopes = scopeOfApplicationMapper.selectList(new LambdaQueryWrapper<SysFloatingColumnSchemeScopeOfApplication>().ne(SysFloatingColumnSchemeScopeOfApplication::getFloatingBarSchemeId, schemeId).inSql(SysFloatingColumnSchemeScopeOfApplication::getFloatingBarSchemeId, sqlInCondition));

        // 检查是否有重叠
        for (SysFloatingColumnSchemeScopeOfApplication enabledScope : enabledScopes) {
            Set<Integer> enabledScopeIds = Arrays.stream(enabledScope.getApplicationScopeId().split(",")).map(Integer::parseInt).collect(Collectors.toSet());

            if (!Collections.disjoint(currentScopeIds, enabledScopeIds)) {
//                System.err.println("--------------->>>>重叠的范围ID: " + currentScopeIds.stream());
                return true; // 存在重叠
            }
        }

        return false; // 无重叠
    }

    /**
     * 生成冲突提示信息，展示冲突的栏目名称。
     *
     * @param schemeId 悬浮栏方案ID
     * @return 冲突提示信息
     */
    /**
     * 生成关于悬浮栏方案范围冲突的提示信息。
     * 只有当尝试启用的悬浮栏方案与现有启用的方案存在范围重叠时，才会生成冲突信息。
     *
     * @param schemeId     当前操作的悬浮栏方案ID。
     * @param siteId       站点ID，当前示例中未直接使用，根据需求可能用于扩展功能。
     * @param enableAction 操作类型，0 表示启用操作，其他值视为未知操作。
     * @return 包含冲突范围栏目的提示信息，如果没有冲突则返回空字符串。
     */
    private String generateConflictMessage(Integer schemeId, Integer siteId, Integer enableAction) {
        // 确认当前悬浮栏方案是否与其他方案存在范围重叠
        if (!hasOverlapWithOtherEnabled(schemeId, siteId)) {
            return ""; // 若无重叠，直接返回空字符串表示无冲突
        }

        // 获取与当前悬浮栏方案ID相关的冲突范围ID集合
        List<SysFloatingColumnSchemeScopeOfApplication> conflictingScopes = scopeOfApplicationMapper.selectList(new LambdaQueryWrapper<SysFloatingColumnSchemeScopeOfApplication>().eq(SysFloatingColumnSchemeScopeOfApplication::getFloatingBarSchemeId, schemeId));

        // 将范围ID字符串转换为整数集合
        Set<Integer> conflictScopeIds = conflictingScopes.stream().flatMap(scope -> Arrays.stream(scope.getApplicationScopeId().split(","))).map(Integer::parseInt).collect(Collectors.toSet());

        //查询当前已经启用的悬浮栏方案下的的应用范围跟当前设置的范围有重复的就只提出出来重复的id
        conflictScopeIds.retainAll(checkBoundByColumnId(conflictScopeIds, siteId));

        // 根据冲突范围ID查询对应的栏目名称
        List<String> conflictingScopeNames = columnMapper.selectList(new LambdaQueryWrapper<SysColumn>().in(SysColumn::getId, conflictScopeIds)).stream().map(SysColumn::getColumnName).collect(Collectors.toList());

        // 根据操作类型生成具体的提示信息
        if (enableAction == 0) { // 启用操作
            // 拼接冲突范围名称，提醒用户需先解决冲突
            return "栏目：“" + String.join("”， “", conflictingScopeNames) + "”已设置有悬浮栏，请先移除相关设置再尝试启用。";
        } else {
            return "未知的操作类型";
        }
    }

    private Set<Integer> checkBoundByColumnId(Set<Integer> columnIds, Integer siteId) {
        // 构造SQL查询条件，用于查找所有启用的悬浮栏方案及其范围
        /*String sqlInCondition = columnIds.stream().map(Object::toString).collect(Collectors.joining(","));*/ // 移除了外部的括号

        List<SysFloatingColumnSchemeScopeOfApplication> enabledConflictingScopes = scopeOfApplicationMapper.selectList(new LambdaQueryWrapper<SysFloatingColumnSchemeScopeOfApplication>().inSql(SysFloatingColumnSchemeScopeOfApplication::getFloatingBarSchemeId, "SELECT id FROM sys_floating_column_scheme WHERE is_disable = 0 AND site_id = " + siteId).in(SysFloatingColumnSchemeScopeOfApplication::getApplicationScopeId, columnIds));

        // 转换为范围ID集合
        Set<Integer> duplicateScopeIds = new HashSet<>();
        for (SysFloatingColumnSchemeScopeOfApplication scope : enabledConflictingScopes) {
            duplicateScopeIds.addAll(Arrays.stream(scope.getApplicationScopeId().split(",")).map(Integer::parseInt).collect(Collectors.toSet()));
        }

        return duplicateScopeIds;
    }


    // 新增方法：统计当前站点下已启用的全局悬浮栏数量
    public Integer countGlobalEnabled(Integer siteId) {
        return baseMapper.selectCount(new LambdaQueryWrapper<SysFloatingColumnScheme>().eq(SysFloatingColumnScheme::getSiteId, siteId).eq(SysFloatingColumnScheme::getIsGlobal, 1).eq(SysFloatingColumnScheme::getIsDisable, false));
    }


    private String getColumnNameById(String overlapId) {
        return columnMapper.getColumnNamesById(overlapId);
    }
}
