package com.huoran.iasf.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.entity.ProductDetails;
import com.huoran.iasf.entity.SysColumn;
import com.huoran.iasf.mapper.SysColumnMapper;
import com.huoran.iasf.mapper.SysProductDetailsMapper;
import com.huoran.iasf.service.SysProductDetailsService;
import com.huoran.iasf.vo.DisciplineAndSpecialtyVO;
import com.huoran.iasf.vo.req.PageCourseProductReq;
import com.huoran.iasf.vo.resp.PageCourseProductRespVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 综合产品与课程详情信息表 服务实现类
 * </p>
 *
 * @author cheney
 * @since 2024-06-12
 */
@Service
public class SysProductDetailsServiceImpl extends ServiceImpl<SysProductDetailsMapper, ProductDetails> implements SysProductDetailsService {

    @Autowired
    private SysColumnMapper columnMapper;
    @Autowired
    private StringRedisTemplate redisTemplate;

    @Override
    public boolean existsByProductName(String name, Integer siteId) {
        if (this.count(new QueryWrapper<ProductDetails>().eq("name", name).eq("site_id", siteId)) > 0) {
            return true;
        }
        return false;
    }

    @Override
    public boolean existsOtherByProductName(String name, Integer currentId, Integer siteId) {
        if (this.count(new QueryWrapper<ProductDetails>().eq("name", name).ne("id", currentId).eq("site_id", siteId)) > 0) {
            return true;
        }
        return false;
    }

    @Override
    public Integer statisticsOfPageViews(Integer id) {
        String key = "courseProductPreview:contentId:" + id;
        //查看缓存是否有数据
        String returnValue = redisTemplate.opsForValue().get(key);
        if (returnValue == null) {
            returnValue = "0";
        }
        returnValue = Integer.valueOf(returnValue) + 1 + "";
        //保存到缓存中
        redisTemplate.opsForValue().set(key, returnValue + "");

        ProductDetails sysContent = new ProductDetails();
        sysContent.setId(id);
        sysContent.setTotalBrowsing(Integer.valueOf(returnValue));
        baseMapper.updateById(sysContent);

        return Integer.valueOf(returnValue);
    }

    @Override
    public R courseProduct(PageCourseProductReq reqVO) {

        /*// 当栏目ID集合不为空且有特定模板ID需求时，调用相应方法获取分页列表
        if (!ObjectUtil.isEmpty(reqVO.getColumnId())) {
            SysColumn column = columnMapper.selectById(reqVO.getColumnId());
            if (!ObjectUtil.isEmpty(column) && column.getTemplateId() == 12) {
                // 根据模板ID为12的特定情况调用相应方法
                IPage<PageContentRespVO> pageList = productDetailsMapper.productCourseList(page, reqVO);
                return R.success(pageList);
            }
        }*/

        // 初始化分页参数
        Page<PageCourseProductRespVO> page = new Page<>(reqVO.getPageNum(), reqVO.getPageSize());
        // 默认情况下调用baseMapper获取分页列表
        IPage<PageCourseProductRespVO> pageList = baseMapper.courseProduct(page, reqVO);

        //查询类型/标签
        pageList.getRecords().forEach(item -> {
            item.setTag(baseMapper.queryTypeByCategoryId(item.getCategoryId()));
        });


        //全部：就是展示未删除的且已发布的
        //


        return R.success(pageList);
    }

    @Override
    public ProductDetails foregroundPreview(Integer id) {
        //预览增加次数
        statisticsOfPageViews(id);


        DisciplineAndSpecialtyVO disciplineAndSpecialtyVO = baseMapper.queryDisciplineAndSpecialty(id);
        ProductDetails productDetails = baseMapper.foregroundPreview(id);
        //查看栏目名称
        SysColumn column = columnMapper.selectById(productDetails.getColumnId());
        productDetails.setColumnName(column.getColumnName());
        if (ObjectUtil.isNotEmpty(productDetails)) {
            if (ObjectUtil.isNotEmpty(disciplineAndSpecialtyVO)) {
                productDetails.setDisciplineName(disciplineAndSpecialtyVO.getDisciplineName());
                productDetails.setProfessionalClassName(disciplineAndSpecialtyVO.getProfessionalClassName());
                productDetails.setProfessionalName(disciplineAndSpecialtyVO.getProfessionalName());
            }
        }
        return productDetails;
    }


}
